#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.translations import translations

translations.define("woost.extensions.blocks.slots.blocks",
    ca = u"Blocs",
    es = u"Bloques",
    en = u"Blocks"
)

# BlocksPage
#------------------------------------------------------------------------------
translations.define("BlocksPage",
    ca = u"Pàgina de blocs",
    es = u"Página de bloques",
    en = u"Blocks page"
)

translations.define("BlocksPage-plural",
    ca = u"Pàgines de blocs",
    es = u"Páginas de bloques",
    en = u"Blocks pages"
)

translations.define("BlocksPage.blocks",
    ca = u"Blocs",
    es = u"Bloques",
    en = u"Blocks"
)

# Blocks page template
#------------------------------------------------------------------------------
translations.define("woost.extensions.blocks.blocks_page_template.title",
    ca = u"Plantilla pàgina de blocs",
    es = u"Plantilla página de bloques",
    en = u"Blocks page template"
)

# Block
#------------------------------------------------------------------------------
translations.define("Block",
    ca = u"Bloc",
    es = u"Bloque",
    en = u"Block"
)

translations.define("Block-plural",
    ca = u"Blocs",
    es = u"Bloques",
    en = u"Blocks"
)

translations.define("Block.content",
    ca = u"Contingut",
    es = u"Contenido",
    en = u"Content"
)

translations.define("Block.title",
    ca = u"Títol",
    es = u"Título",
    en = u"Title"
)

translations.define("Block.title-explanation",
    ca = u"Descripció del bloc, d'ús intern pels editors",
    es = u"Descripción del bloque, de uso interno para los editores",
    en = u"Description for this block, used internally by editors"
)

translations.define("Block.heading",
    ca = u"Títol de capçalera",
    es = u"Título de cabecera",
    en = u"Heading"
)

translations.define("Block.heading-explanation",
    ca = u"Títol del bloc, tal com es mostrarà al web",
    es = u"Título del bloque, tal como se mostraré en la web",
    en = u"Block title, as shown on the website"
)

translations.define("Block.html_id",
    ca = u"ID de l'element HTML",
    es = u"ID del elemento HTML",
    en = u"HTML element ID"
)

translations.define("Block.css_class",
    ca = u"Classes CSS",
    es = u"Clases CSS",
    en = u"CSS classes"
)

translations.define("Block.enabled",
    ca = u"Actiu",
    es = u"Activo",
    en = u"Active"
)

translations.define("Block.containers",
    ca = u"Contenidors",
    es = u"Contenedores",
    en = u"Containers"
)

# ImageGalleryBlock
#------------------------------------------------------------------------------
translations.define("ImageGalleryBlock",
    ca = u"Galeria d'imatges",
    es = u"Galería de imágenes",
    en = u"Image gallery"
)

translations.define("ImageGalleryBlock.gallery_type",
    ca = u"Tipus de galeria",
    es = u"Tipo de galería",
    en = u"Gallery type"
)

translations.define("ImageGalleryBlock.gallery_type-thumbnails",
    ca = u"Miniatures",
    es = u"Miniaturas",
    en = u"Thumbnails"
)

translations.define("ImageGalleryBlock.gallery_type-slideshow",
    ca = u"Passador",
    es = u"Pasador",
    en = u"Slideshow"
)

translations.define("ImageGalleryBlock.images",
    ca = u"Imatges",
    es = u"Imágenes",
    en = u"Images"
)

translations.define("ImageGalleryBlock-plural",
    ca = u"Galeries",
    es = u"Galerías",
    en = u"Image galleries"
)

translations.define("ImageGalleryBlock.thumbnail_factory",
    ca = u"Processat per les miniatures",
    es = u"Procesado para las miniaturas",
    en = u"Thumbnail factory"
)

translations.define("ImageGalleryBlock.close_up_factory",
    ca = u"Processat per les imatges ampliades",
    es = u"Procesado para la imagen ampliada",
    en = u"Close up factory"
)

translations.define("ImageGalleryBlock.auto_play",
    ca = u"Transicions automàtiques",
    es = u"Transiciones automáticas",
    en = u"Auto play"
)

translations.define("ImageGalleryBlock.labels_visible",
    ca = u"Mostrar els títols de les imatges",
    es = u"Mostrar los títulos de las imágenes",
    en = u"Show image titles"
)

translations.define("ImageGalleryBlock.footnotes_visible",
    ca = u"Mostrar els peus de foto",
    es = u"Mostrar los pies de foto",
    en = u"Include footnotes"
)

translations.define("ImageGalleryBlock.original_link_visible",
    ca = u"Mostrar un enllaç a la imatge completa",
    es = u"Mostrar un enlace a la imagen completa",
    en = u"Include a link to the original image"
)

translations.define("ImageGalleryBlock.close_up_enabled",
    ca = u"Habilitar la versió ampliada de les imatges",
    es = u"Habilitar la versión ampliada de las imágenes",
    en = u"Enable image close ups"
)

translations.define("ImageGalleryBlock.close_up_preload",
    ca = u"Precàrrega de la versió ampliada de les imatges",
    es = u"Precarga de la versión ampliada de las imágenes",
    en = u"Preload image close ups"
)

translations.define("ImageGalleryBlock.close_up_preload-explanation",
    ca = u"Habilitar per tal que el navegador carregui la versió ampliada de "
         u"les imatges tan bon punt es mostri la galeria. Això accelera la "
         u"navegació, però representa un major impacte pel servidor.",
    es = u"Habilitar para que el navegador empiece a cargar la versión "
         u"ampliada de las imágenes desde el primer momento en que se muestre "
         u"la galería. Esto acelera la navegación, pero representa un mayor "
         u"impacto para el servidor.",
    en = u"Enable to instruct the browser to start downloading the image "
         u"close ups as soon as the gallery is displayed. This improves the "
         u"user experience but causes a higher load for the web server."
)

# ContainerBlock
#------------------------------------------------------------------------------
translations.define("ContainerBlock",
    ca = u"Contenidor",
    es = u"Contenedor",
    en = u"Container"
)

translations.define("ContainerBlock-plural",
    ca = u"Contenidors",
    es = u"Contenedores",
    en = u"Containers"
)

translations.define("ContainerBlock.blocks",
    ca = u"Blocs fills",
    es = u"Bloques hijos",
    en = u"Child blocks"
)

# SlideShowBlock
#------------------------------------------------------------------------------
translations.define("SlideShowBlock",
    ca = u"Passador",
    es = u"Pasador",
    en = u"Slide show"
)

translations.define("SlideShowBlock-plural",
    ca = u"Passadors",
    es = u"Pasadores",
    en = u"Slide shows"
)

translations.define("SlideShowBlock.transition_settings",
    ca = u"Transicions",
    es = u"Transiciones",
    en = u"Transitions"
)

translations.define("SlideShowBlock.autoplay",
    ca = u"Transicions automàtiques",
    es = u"Transiciones automáticas",
    en = u"Autoplay"
)

translations.define("SlideShowBlock.navigation_controls",
    ca = u"Mostrar controls de navegació",
    es = u"Mostrar controles de navegación",
    en = u"Show navigation controls"
)

translations.define("SlideShowBlock.interval",
    ca = u"Freqüència de les transicions",
    es = u"Frecuencia de las transiciones",
    en = u"Time between transitions"
)

translations.define("SlideShowBlock.interval-explanation",
    ca = u"Si s'activen les transicions automàtiques, indica el nombre de "
         u"milisegons abans que el bloc passi a la següent diapositiva.",
    es = u"Si se activan las transiciones automáticas, indica el número de "
         u"milisegundos antes que el bloque pase a la diapositiva siguiente.",
    en = u"If autoplay is enabled, indicates the number of milliseconds that "
         u"pass between automatic transitions."
)

translations.define("SlideShowBlock.transition_duration",
    ca = u"Duració de les transicions",
    es = u"Duración de las transiciones",
    en = u"Transition duration"
)

translations.define("SlideShowBlock.interval-explanation",
    ca = u"Especifica la duració de l'efecte de transició, en milisegons",
    es = u"Especifica la duración del efecto de transición, en milisegundos",
    en = u"Sets the duration of the slide transition effect, in milliseconds"
)

# BannerBlock
#------------------------------------------------------------------------------
translations.define("BannerBlock",
    ca = u"Banner",
    es = u"Banner",
    en = u"Banner"
)

translations.define("BannerBlock-plural",
    ca = u"Banners",
    es = u"Banners",
    en = u"Banners"
)

translations.define("BannerBlock.text",
    ca = u"Text",
    es = u"Texto",
    en = u"Text"
)

translations.define("BannerBlock.text-explanation",
    ca = u"En cas de no especificar-se s'utilitzarà el títol de l'element "
         u"destí",
    es = u"En caso de no especificarse se utilitzará el título del "
         u"elemento destino",
    en = u"If no text is provided the title from the target item will be used"
)

translations.define("BannerBlock.target",
    ca = u"Destí",
    es = u"Destino",
    en = u"Target"
)

translations.define("BannerBlock.image",
    ca = u"Imatge",
    es = u"Imagen",
    en = u"Image"
)

translations.define("BannerBlock.image_factory",
    ca = u"Processat d'imatge",
    es = u"Procesado de imagen",
    en = u"Image processing"
)

translations.define("BannerBlock.label_displayed",
    ca = u"Mostrar el text",
    es = u"Mostrar el texto",
    en = u"Label displayed"
)

translations.define("BannerBlock.label_displayed-explanation",
    ca = u"Si es desactiva aquesta opció no es mostra el text del bloc, "
         u"només la seva imatge.",
    es = u"Si se desactiva esta opción no se muestra el texto del bloque, "
         u"solo su imagen.",
    en = u"If this option is disabled, the block's text won't be displayed, "
         u"only its image will be shown."
)

# MenuBlock
#------------------------------------------------------------------------------
translations.define("MenuBlock",
    ca = u"Menú",
    es = u"Menú",
    en = u"Menu"
)

translations.define("MenuBlock-plural",
    ca = u"Menús",
    es = u"Menús",
    en = u"Menus"
)

translations.define("MenuBlock.root",
    ca = u"Arrel",
    es = u"Raiz",
    en = u"Root"
)

translations.define("MenuBlock.root_visible",
    ca = u"Arrel visible",
    es = u"Raiz visible",
    en = u"Root visible"
)

translations.define("MenuBlock.max_depth",
    ca = u"Profunditat màxima",
    es = u"Profundidad máxima",
    en = u"Maximum depth"
)

translations.define("MenuBlock.expanded",
    ca = u"Expandit",
    es = u"Expandido",
    en = u"Expanded"
)

# RichTextBlock
#------------------------------------------------------------------------------
translations.define("RichTextBlock",
    ca = u"Text",
    es = u"Texto",
    en = u"Text"
)

translations.define("RichTextBlock-plural",
    ca = u"Textos",
    es = u"Textos",
    en = u"Texts"
)

translations.define("RichTextBlock.text",
    ca = u"Text",
    es = u"Texto",
    en = u"Text"
)

translations.define("RichTextBlock.attachments",
    ca = u"Adjunts",
    es = u"Adjuntos",
    en = u"Attachments"
)

# TranslatedRichTextBlock
#------------------------------------------------------------------------------
translations.define("TranslatedRichTextBlock",
    ca = u"Text traduïble",
    es = u"Texto traducible",
    en = u"Translated text"
)

translations.define("TranslatedRichTextBlock-plural",
    ca = u"Textos traduïbles",
    es = u"Textos traducibles",
    en = u"Translated texts"
)

translations.define("TranslatedRichTextBlock.text",
    ca = u"Text",
    es = u"Texto",
    en = u"Text"
)

translations.define("TranslatedRichTextBlock.text",
    ca = u"Text",
    es = u"Texto",
    en = u"Text"
)

translations.define("TranslatedRichTextBlock.image",
    ca = u"Imatge",
    es = u"Imagen",
    en = u"Image"
)

translations.define("TranslatedRichTextBlock.image_factory",
    ca = u"Processat de la imatge",
    es = u"Procesado de la imagen",
    en = u"Image processing"
)

translations.define("TranslatedRichTextBlock.attachments",
    ca = u"Adjunts",
    es = u"Adjuntos",
    en = u"Attachments"
)

# VimeoBlock
#------------------------------------------------------------------------------
translations.define("VimeoBlock",
    ca = u"Vídeo de Vimeo",
    es = u"Video de Vimeo",
    en = u"Vimeo video"
)

translations.define("VimeoBlock-plural",
    ca = u"Vídeos de Vimeo",
    es = u"Vídeos de Vimeo",
    en = u"Vimeo videos"
)

translations.define("VimeoBlock.video",
    ca = u"Vídeo",
    es = u"Video",
    en = u"Video"
)

translations.define("VimeoBlock.video_width",
    ca = u"Amplada",
    es = u"Ancho",
    en = u"Width"
)

translations.define("VimeoBlock.video_width-explanation",
    ca = u"en píxels",
    es = u"en píxeles",
    en = u"in pixels"
)

translations.define("VimeoBlock.video_height",
    ca = u"Alçada",
    es = u"Alto",
    en = u"Height"
)

translations.define("VimeoBlock.video_height-explanation",
    ca = u"en píxels",
    es = u"en píxeles",
    en = u"in pixels"
)

# TwitterTimelineBlock
#------------------------------------------------------------------------------
translations.define("TwitterTimelineBlock",
    ca = u"Missatges de Twitter",
    es = u"Mensajes de Twitter",
    en = u"Twitter timeline"
)

translations.define("TwitterTimelineBlock-plural",
    ca = u"Missatges de Twitter",
    es = u"Mensajes de Twitter",
    en = u"Twitter timeline"
)

translations.define("TwitterTimelineBlock.account",
    ca = u"Compte de Twitter",
    es = u"Cuenta de Twitter",
    en = u"Twitter account"
)

translations.define("TwitterTimelineBlock.max_tweets",
    ca = u"Missatges a mostrar",
    es = u"Mensajes a mostrar",
    en = u"Max tweets"
)

# LinksBox
#------------------------------------------------------------------------------
translations.define("LinksBlock",
    ca = u"Llista d'enllaços",
    es = u"Lista de enlaces",
    en = u"Link list"
)

translations.define("LinksBlock-plural",
    ca = u"Llistes d'enllaços",
    es = u"Listas de enlaces",
    en = u"Link lists"
)

translations.define("LinksBlock.links",
    ca = u"Enllaços",
    es = u"Enlaces",
    en = u"Links"
)

# FolderBlock
#------------------------------------------------------------------------------
translations.define("FolderBlock",
    ca = u"Llistat de documents fills",
    es = u"Listado de documentos hijos",
    en = u"Child documents listing"
)

translations.define("FolderBlock-plural",
    ca = u"Llistats de documents fills",
    es = u"Listados de documentos hijos",
    en = u"Child documents listings"
)

translations.define("FolderBlock.show_hidden_children",
    ca = u"Mostrar documents ocults",
    es = u"Mostrar documentos ocultos",
    en = u"Show hidden documents"
)

translations.define("FolderBlock.show_thumbnails",
    ca = u"Mostrar miniatures",
    es = u"Mostrar miniaturas",
    en = u"Show thumbnails"
)

translations.define("FolderBlock.thumbnails_factory",
    ca = u"Processador de miniatures",
    es = u"Procesador de miniaturas",
    en = u"Thumbnail factory"
)

# LoginBlock
#------------------------------------------------------------------------------
translations.define("LoginBlock",
    ca = u"Formulari d'autenticació d'usuari",
    es = u"Formulario de autenticación de usuario",
    en = u"Login form"
)

translations.define("LoginBlock-plural",
    ca = u"Formularis d'autenticació d'usuari",
    es = u"Formularios de autenticación de usuario",
    en = u"Login forms"
)

translations.define("LoginBlock.login_target",
    ca = u"Pàgina de destí",
    es = u"Página de destino",
    en = u"Destination page"
)

translations.define("LoginBlock.login_target-explanation",
    ca = u"La pàgina que rebrà la petició d'autenticació de l'usuari",
    es = u"La pàgina que recibirá la petición de autenticación del "
         u"usuario",
    en = u"The page that will handle the user's authentication request"
)

translations.define("LoginBlockForm.user",
    ca = u"Usuari",
    es = u"Usuario",
    en = u"User"
)

translations.define("LoginBlockForm.password",
    ca = u"Contrasenya",
    es = u"Contraseña",
    en = u"Password"
)

translations.define("woost.extensions.blocks.LoginBlockView.submit_button",
    ca = u"Entrar",
    es = u"Entrar",
    en = u"Login"
)

# IFrameBlock
#------------------------------------------------------------------------------
translations.define("IFrameBlock",
    ca = u"IFrame",
    es = u"IFrame",
    en = u"IFrame"
)

translations.define("IFrameBlock-plural",
    ca = u"IFrames",
    es = u"IFrames",
    en = u"IFrames"
)

translations.define("IFrameBlock.src",
    ca = u"Adreça a mostrar",
    es = u"Dirección a mostrar",
    en = u"Content URL"
)

translations.define("IFrameBlock.width",
    ca = u"Amplada",
    es = u"Ancho",
    en = u"Width"
)

translations.define("IFrameBlock.height",
    ca = u"Alçada",
    es = u"Alto",
    en = u"Height"
)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from decimal import Decimal
from cocktail.modeling import getter
from cocktail import schema
from cocktail.translations import translations
from woost.models import (
    Publishable,
    Document,
    Controller,
    Template,
    File
)


class ECommerceProduct(Publishable):

    instantiable = False

    members_order = [
        "description",
        "price",
        "weight",
        "attachments",
        "purchase_model",
        "purchases",
        "template"
    ]

    default_controller = schema.DynamicDefault(
        lambda: Controller.get_instance(
            qname = "woost.extensions.ecommerce.product_controller"
        )
    )

    description = schema.String(
        translated = True,
        edit_control = "woost.views.RichTextEditor",
        member_group = "product_data"
    )

    price = schema.Decimal(
        required = True,
        default = Decimal("0"),
        member_group = "product_data"
    )

    weight = schema.Decimal(
        translate_value = lambda value, language = None, **kwargs:
            "" if not value else "%s Kg" % translations(value, language),
        member_group = "product_data"
    )

    attachments = schema.Collection(
        items = schema.Reference(type = File),
        related_end = schema.Collection(),
        member_group = "product_data"
    )

    purchase_model = schema.Reference(
        class_family = "woost.extensions.ecommerce.ecommercepurchase."
                       "ECommercePurchase",
        default = schema.DynamicDefault(
            lambda: ECommerceProduct.purchase_model.class_family
        ),
        required = True,
        searchable = False,
        member_group = "product_data"
    )

    purchases = schema.Collection(
        items = "woost.extensions.ecommerce.ecommercepurchase."
                "ECommercePurchase",
        bidirectional = True,
        visible = False,
        member_group = "product_data"
    )

    template = schema.Reference(
        type = Template,
        related_end = schema.Collection(),
        default = schema.DynamicDefault(
            lambda: Template.get_instance(
                qname = "woost.extensions.ecommerce.product_template"
            )
        ),
        member_group = "presentation"
    )

    def get_image(self):
        for attachment in self.attachments:
            if attachment.resource_type == "image" \
            and attachment.is_accessible():
                return attachment

    def offers(self):
        from woost.extensions.ecommerce import ECommerceExtension
        for pricing in ECommerceExtension.instance.pricing:
            if not pricing.hidden and pricing.applies_to(self):
                yield pricing

    @getter
    def inherited_resources(self):

        if self.inherit_resources and self.parent is None:
            catalog = Document.get_instance(
                qname = "woost.extensions.ecommerce.catalog_page"
            )

            if catalog:
                for resource in catalog.inherited_resources:
                    yield resource

                for resource in catalog.branch_resources:
                    yield resource
        else:
            for resource in Publishable.inherited_resources.__get__(self):
                yield resource




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from cocktail.translations import translations

translations.define("Country",
    ca = u"País",
    es = u"País",
    en = u"Country"
)

translations.define("Country-plural",
    ca = u"Paisos",
    es = u"Países",
    en = u"Country"
)

translations.define("Country.country_name",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

translations.define("Country.iso_code",
    ca = u"Codi ISO",
    es = u"Código ISO",
    en = u"ISO code"
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

# OpenGraphExtension
#------------------------------------------------------------------------------
translations.define("OpenGraphExtension.categories",
    ca = u"Categories",
    es = u"Categorías",
    en = u"Categories"
)

# Extensions to the Publishable model
#------------------------------------------------------------------------------
translations.define("Publishable.open_graph",
    ca = u"Integració amb OpenGraph/Facebook",
    es = u"Integración con OpenGraph/Facebook",
    en = u"OpenGraph/Facebook integration"
)

translations.define("Publishable.open_graph_enabled",
    ca = u"Publicar metadades per OpenGraph/Facebook",
    es = u"Publicar metadatos para OpenGraph/Facebook",
    en = u"Publish OpenGraph/Facebook metadata"
)

translations.define("Publishable.open_graph_type",
    ca = u"Tipus d'objecte",
    es = u"Tipo de objeto",
    en = u"Object type"
)

# OpenGraphCategory
#------------------------------------------------------------------------------
translations.define("OpenGraphCategory",
    ca = u"Categoria d'objecte OpenGraph",
    es = u"Categoría de objeto OpenGraph",
    en = u"OpenGraph object category"
)

translations.define("OpenGraphCategory-plural",
    ca = u"Categories d'objecte OpenGraph",
    es = u"Categorías de objeto OpenGraph",
    en = u"OpenGraph object categories"
)

translations.define("OpenGraphCategory.title",
    ca = u"Títol",
    es = u"Título",
    en = u"Title"
)

translations.define("OpenGraphCategory.code",
    ca = u"Codi",
    es = u"Code",
    en = u"Código"
)

translations.define("OpenGraphCategory.types",
    ca = u"Tipus",
    es = u"Tipos",
    en = u"Types"
)

# OpenGraphType
#------------------------------------------------------------------------------
translations.define("OpenGraphType",
    ca = u"Tipus d'objecte OpenGraph",
    es = u"Tipo de objeto OpenGraph",
    en = u"OpenGraph object type"
)

translations.define("OpenGraphType-plural",
    ca = u"Tipus d'objecte OpenGraph",
    es = u"Tipos de objeto OpenGraph",
    en = u"OpenGraph object types"
)

translations.define("OpenGraphType.title",
    ca = u"Títol",
    es = u"Título",
    en = u"Title"
)

translations.define("OpenGraphType.code",
    ca = u"Codi",
    es = u"Código",
    en = u"Code"
)

translations.define("OpenGraphType.category",
    ca = u"Categoria",
    es = u"Categoría",
    en = u"Category"
)

# Category and type titles
#------------------------------------------------------------------------------

# Activities
translations.define("opengraph.categories.activities",
    ca = u"Activitats",
    es = u"Actividades",
    en = u"Activities"
)

translations.define("opengraph.types.activity",
    ca = u"Activitat",
    es = u"Actividad",
    en = u"Activity"
)

translations.define("opengraph.types.sport",
    ca = u"Esport",
    es = u"Deporte",
    en = u"Sport"
)

# Businesses
translations.define("opengraph.categories.businesses",
    ca = u"Negocis",
    es = u"Negocios",
    en = u"Businesses"
)

translations.define("opengraph.types.bar",
    ca = u"Bar",
    es = u"Bar",
    en = u"Bar"
)

translations.define("opengraph.types.company",
    ca = u"Companyia",
    es = u"Compañía",
    en = u"Company"
)

translations.define("opengraph.types.cafe",
    ca = u"Cafè",
    es = u"Café",
    en = u"Cafe"
)

translations.define("opengraph.types.hotel",
    ca = u"Hotel",
    es = u"Hotel",
    en = u"Hotel"
)

translations.define("opengraph.types.restaurant",
    ca = u"Restaurant",
    es = u"Restaurante",
    en = u"Restaurant"
)

# Groups
translations.define("opengraph.categories.groups",
    ca = u"Grups",
    es = u"Grupos",
    en = u"Groups"
)

translations.define("opengraph.types.cause",
    ca = u"Causa",
    es = u"Causa",
    en = u"Cause"
)

translations.define("opengraph.types.sports_league",
    ca = u"Lliga esportiva",
    es = u"Liga deportiva",
    en = u"Sports league"
)

translations.define("opengraph.types.sports_team",
    ca = u"Equip esportiu",
    es = u"Equipo deportivo",
    en = u"Sports team"
)

# Organizations
translations.define("opengraph.categories.organizations",
    ca = u"Organitzacions",
    es = u"Organizaciones",
    en = u"Organizations"
)

translations.define("opengraph.types.band",
    ca = u"Grup musical",
    es = u"Grupo musical",
    en = u"Band"
)

translations.define("opengraph.types.government",
    ca = u"Govern",
    es = u"Gobierno",
    en = u"Government"
)

translations.define("opengraph.types.non_profit",
    ca = u"ONG",
    es = u"ONG",
    en = u"NGO"
)

translations.define("opengraph.types.school",
    ca = u"Escola",
    es = u"Escuela",
    en = u"School"
)

translations.define("opengraph.types.university",
    ca = u"Universitat",
    es = u"Universidad",
    en = u"University"
)

# People
translations.define("opengraph.categories.people",
    ca = u"Gent",
    es = u"Gente",
    en = u"People"
)

translations.define("opengraph.types.actor",
    ca = u"Actor",
    es = u"Actor",
    en = u"Actor"
)

translations.define("opengraph.types.athlete",
    ca = u"Atleta",
    es = u"Atleta",
    en = u"Athlete"
)

translations.define("opengraph.types.author",
    ca = u"Autor",
    es = u"Autor",
    en = u"Author"
)

translations.define("opengraph.types.director",
    ca = u"Director de cine",
    es = u"Director de cine",
    en = u"Director"
)

translations.define("opengraph.types.musician",
    ca = u"Músic",
    es = u"Músico",
    en = u"Musician"
)

translations.define("opengraph.types.politician",
    ca = u"Polític",
    es = u"Político",
    en = u"Politician"
)

translations.define("opengraph.types.profile",
    ca = u"Perfil",
    es = u"Perfil",
    en = u"Profile"
)

translations.define("opengraph.types.public_figure",
    ca = u"Figura pública",
    es = u"Figura pública",
    en = u"Public figure"
)

# Places
translations.define("opengraph.categories.places",
    ca = u"Llocs",
    es = u"Sitios",
    en = u"Places"
)

translations.define("opengraph.types.city",
    ca = u"Ciutat",
    es = u"Ciudad",
    en = u"City"
)

translations.define("opengraph.types.country",
    ca = u"País",
    es = u"País",
    en = u"Country"
)

translations.define("opengraph.types.landmark",
    ca = u"Punt d'interès",
    es = u"Punto de interés",
    en = u"Landmark"
)

translations.define("opengraph.types.state_province",
    ca = u"Província o regió",
    es = u"Provincia o región",
    en = u"State province"
)

# Products and entertainment
translations.define("opengraph.categories.products_and_entertainment",
    ca = u"Productes i entreteniment",
    es = u"Productos y entretenimiento",
    en = u"Products and entertainment"
)

translations.define("opengraph.types.album",
    ca = u"Àlbum",
    es = u"Álbum",
    en = u"Album"
)

translations.define("opengraph.types.book",
    ca = u"Llibre",
    es = u"Libro",
    en = u"Book"
)

translations.define("opengraph.types.drink",
    ca = u"Beguda",
    es = u"Bebida",
    en = u"Drink"
)

translations.define("opengraph.types.food",
    ca = u"Menjar",
    es = u"Comida",
    en = u"Food"
)

translations.define("opengraph.types.game",
    ca = u"Joc",
    es = u"Juego",
    en = u"Game"
)

translations.define("opengraph.types.movie",
    ca = u"Pel·lícula",
    es = u"Película",
    en = u"Movie"
)

translations.define("opengraph.types.product",
    ca = u"Producte",
    es = u"Producto",
    en = u"Product"
)

translations.define("opengraph.types.song",
    ca = u"Cançó",
    es = u"Canción",
    en = u"Song"
)

translations.define("opengraph.types.tv_show",
    ca = u"Programa de televisió",
    es = u"Programa de televisión",
    en = u"TV show"
)

# Websites
translations.define("opengraph.categories.websites",
    ca = u"Llocs web",
    es = u"Sitios web",
    en = u"Web sites"
)

translations.define("opengraph.types.article",
    ca = u"Article",
    es = u"Artículo",
    en = u"Article"
)

translations.define("opengraph.types.blog",
    ca = u"Blog",
    es = u"Blog",
    en = u"Blog"
)

translations.define("opengraph.types.website",
    ca = u"Lloc web",
    es = u"Sitio web",
    en = u"Web site"
)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			December 2008
"""
from cocktail.modeling import extend, call_base
from cocktail.translations import translations
from cocktail.html import Element, templates
from cocktail.html.databoundcontrol import data_bound
from woost.models import (
    Item,
    CreatePermission,
    ModifyPermission,
    DeletePermission,
    get_current_user
)


class ItemSelector(Element):

    value = None
    _empty_label = None
    existing_items_only = False

    empty_label = ""

    def _build(self):

        Element._build(self)
        data_bound(self)

        self.add_resource("/resources/scripts/ItemSelector.js")
        self.set_client_param("emptyLabel", self.empty_label)

        self.input = templates.new("cocktail.html.HiddenInput")
        self.append(self.input)
        self.binding_delegate = self.input

        self.selection_label = templates.new("woost.views.ItemLabel")
        self.selection_label.tag = "span"
        self.selection_label.add_class("selection_label")
        self.append(self.selection_label)

        self.buttons = self.create_buttons()
        self.append(self.buttons)

    def _ready(self):

        Element._ready(self)

        if self.member:

            if self.data_display:
                self._param_name = self.data_display.get_member_name(
                    self.member,
                    self.language
                )
            else:
                self._param_name = self.member.name

            if self.existing_items_only or not self.member.integral:
                # Select
                self.select_button = self.create_select_button()
                self.buttons.append(self.select_button)

            if not self.existing_items_only:

                user = get_current_user()

                if self.member.integral:

                    if self.value is None:
                        # New
                        if any(
                            user.has_permission(CreatePermission, target = cls)
                            for cls in self.member.type.schema_tree()
                        ):
                            self.new_button = self.create_new_button()
                            self.buttons.append(self.new_button)
                    else:
                        # Edit
                        if any(
                            user.has_permission(ModifyPermission, target = cls)
                            for cls in self.member.type.schema_tree()
                        ):
                            self.edit_button = self.create_edit_button()
                            self.buttons.append(self.edit_button)

                        # Delete
                        if any(
                            user.has_permission(DeletePermission, target = cls)
                            for cls in self.member.type.schema_tree()
                        ):
                            self.delete_button = self.create_delete_button()
                            self.buttons.append(self.delete_button)

                elif self.value is not None:

                    # Edit
                    if any(
                        user.has_permission(ModifyPermission, target = cls)
                        for cls in self.member.type.schema_tree()
                    ):
                        self.edit_button = self.create_edit_button()
                        self.buttons.append(self.edit_button)

                    # Unlink
                    self.unlink_button = self.create_unlink_button()
                    self.buttons.append(self.unlink_button)

        if self.value is None:
            self.selection_label.add_class("empty_selection")
            self.selection_label.append(self.empty_label)
        else:
            self.input["value"] = self.value.id
            self.selection_label.item = self.value

    def create_buttons(self):
        buttons = Element()
        buttons.add_class("ItemSelector-buttons")
        return buttons

    def create_select_button(self):

        select_button = Element("button",
            name = "ItemSelector-select",
            type = "submit",
            class_name = "ItemSelector-button select",
            value = self.member.type.full_name + "-" + self._param_name
        )
        select_button.append(
            translations("woost.views.ItemSelector select")
        )
        return select_button

    def create_unlink_button(self):

        unlink_button = Element("button",
            name = "ItemSelector-unlink",
            type = "submit",
            class_name = "ItemSelector-button unlink",
            value = self.member.name
        )
        unlink_button.append(
            translations("woost.views.ItemSelector unlink")
        )
        return unlink_button

    def create_new_button(self):

        new_button = Element(class_name = "ItemSelector-button new")

        instantiable_types = set(
            content_type
            for content_type in (
                [self.member.type] + list(self.member.type.derived_schemas())
            )
            if content_type.visible
            and content_type.instantiable
            and get_current_user().has_permission(
                CreatePermission,
                target = content_type
            )
        )

        if len(instantiable_types) > 1:

            new_button.add_class("selector")
            label = Element("span", class_name = "label")
            new_button.append(label)

            container = Element(class_name = "selector_content")
            new_button.append(container)

            content_type_tree = templates.new("woost.views.ContentTypeTree")
            content_type_tree.root = self.member.type
            content_type_tree.filter_item = instantiable_types.__contains__

            @extend(content_type_tree)
            def create_label(tree, content_type):
                label = call_base(content_type)
                label.tag = "button"
                label["type"] = "submit"
                label["name"] = "ItemSelector-new"
                label["value"] = self.member.name + "-" + content_type.full_name
                return label

            container.append(content_type_tree)
        else:
            new_button.tag = "button"
            new_button["type"] = "submit"
            new_button["name"] = "ItemSelector-new"
            new_button["value"] = \
                self.member.name + "-" + list(instantiable_types)[0].full_name
            label = new_button

        label.append(translations("woost.views.ItemSelector new"))

        return new_button

    def create_edit_button(self):

        edit_button = Element("button",
            name = "ItemSelector-edit",
            type = "submit",
            class_name = "ItemSelector-button edit",
            value = self.member.name
        )
        edit_button.append(
            translations("woost.views.ItemSelector edit")
        )
        return edit_button

    def create_delete_button(self):

        delete_button = Element("button",
            name = "ItemSelector-unlink",
            type = "submit",
            class_name = "ItemSelector-button delete",
            value = self.member.name
        )
        delete_button.append(
            translations("woost.views.ItemSelector delete")
        )
        return delete_button




#-*- coding: utf-8 -*-
u"""Defines the `Basket` class.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
from cocktail.persistence import datastore
from cocktail.events import Event
from cocktail.controllers import session
from woost.extensions.ecommerce.ecommerceorder import ECommerceOrder
from woost.extensions.ecommerce.ecommercepurchase import ECommercePurchase
from woost.extensions.ecommerce.ecommerceproduct import ECommerceProduct


class Basket(object):

    session_key = "woost.extensions.ecommerce.basket"

    created = Event(doc = """
        An event triggered on the class when a new instance is created.

        @ivar basket: A reference to the new instance.
        @type basket: L{Basket}
        """)


    @classmethod
    def get(cls, create_new = True):
        """Obtains the shop order for the current user session.

        If the user had not started an order yet, a new one is created.

        :rtype: `~woost.extensions.ecommerce.ecommerceorder.ECommerceOrder`
        """
        order = getattr(
            cherrypy.request,
            "woost_ecommerce_order",
            None
        )

        if order is None:
            order = cls.restore()

            if order is None and create_new:
                order = ECommerceOrder()
                cls.created(basket = order)

            cherrypy.request.woost_ecommerce_order = order

        return order

    @classmethod
    def pop(cls):
        """Remove the current shop order from the session."""
        order = cls.get(create_new = False)

        if order is not None:
            session.pop(cls.session_key, None)
            del cherrypy.request.woost_ecommerce_order

        return order

    @classmethod
    def empty(cls):
        """Removes all products from the basket."""
        order = cls.get(create_new = False)

        if order is not None:
            for purchase in list(order.purchases):
                purchase.delete()
            datastore.commit()

    @classmethod
    def store(cls):

        order = cls.get()
        order.insert()

        for purchase in order.purchases:
            purchase.insert()

        datastore.commit()
        session[cls.session_key] = order.id

    @classmethod
    def restore(cls):
        session_data = session.get(cls.session_key)

        if session_data is None:
            return None
        else:
            return ECommerceOrder.get_instance(session_data)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
from cocktail.html import templates
from woost.views.contentdisplaymixin import ContentDisplayMixin

PropertyTable = templates.get_class("cocktail.html.PropertyTable")


class ContentPropertyTable(ContentDisplayMixin, PropertyTable):

    def __init__(self, *args, **kwargs):
        PropertyTable.__init__(self, *args, **kwargs)
        ContentDisplayMixin.__init__(self)




#-*- coding: utf-8 -*-
u"""Defines the `Block` model.

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.iteration import last
from cocktail.translations import translations
from cocktail import schema
from cocktail.html import templates, Element
from woost.models import Item, Publishable

default_tag = object()


class Block(Item):

    instantiable = False
    collapsed_backoffice_menu = True
    view_class = None
    tag = default_tag

    members_order = [
        "title",
        "heading",
        "html_id",
        "css_class",
        "enabled",
        "containers"
    ]

    groups_order = ["content"]

    title = schema.String(
        descriptive = True,
        member_group = "content"
    )

    heading = schema.String(
        translated = True,
        member_group = "content"
    )

    html_id = schema.String(
        listed_by_default = False,
        member_group = "content"
    )

    css_class = schema.String(
        member_group = "content"
    )

    enabled = schema.Boolean(
        required = True,
        default = True,
        member_group = "content"
    )

    containers = schema.Collection(
        items = "woost.extensions.blocks.containerblock.ContainerBlock",
        bidirectional = True,
        related_key = "blocks",
        editable = False,
        member_group = "content"
    )

    def create_view(self):

        if self.view_class is None:
            raise ValueError("No view specified for block %s" % self)

        view = templates.new(self.view_class)
        self.init_view(view)
        return view

    def init_view(self, view):
        view.block = self
        view.set_client_param("blockId", self.id)

        view.add_class("block")

        if self.html_id:
            view["id"] = self.html_id

        if self.css_class:
            view.add_class(self.css_class)

        view.add_class("block%d" % self.id)

        if self.qname:
            view.add_class(self.qname.replace(".", "-"))

        if self.tag is not default_tag:
            view.tag = self.tag

        if self.heading:
            if hasattr(view, "heading"):
                view.heading = self.heading
            else:
                view.block_heading = Element("h2")
                view.block_heading.append(self.heading)
                view.insert(0, view.block_heading)

    def descend_tree(self, include_self = False):
        if include_self:
            yield self

    def find_publication_points(self):
        """Find all the publishable elements that contain this block, either
        directly or nested within one or more containers.

        :return: An iterable sequence of all the publishable elements that
            contain the block.
        :rtype: `woost.models.Publishable` iterable sequence
        """
        def iter_points(block, path):
            for member in block.__class__.members().itervalues():
                if (
                    isinstance(member, schema.RelationMember)
                    and member.related_type
                    and issubclass(member.related_type, Publishable)
                    and member.related_end.visible
                ):
                    value = block.get(member)
                    if value:
                        if isinstance(member, schema.Reference):
                            yield [value, member] + path
                        else:
                            for publishable in value:
                                yield [publishable, member] + path

            for container in block.containers:
                for point in iter_points(container, [container] + path):
                    yield point

        return iter_points(self, [])




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			August 2009
"""
from cocktail.modeling import getter, cached_getter
from cocktail.schema.expressions import (
    Self,
    Constant,
    InclusionExpression,
    ExclusionExpression,
    IsInstanceExpression,
    IsNotInstanceExpression
)
from cocktail import schema
from cocktail.translations import translations
from cocktail.html import templates
from cocktail.html.datadisplay import MULTIPLE_SELECTION
from cocktail.controllers.userfilter import (
    UserFilter,
    CollectionFilter,
    user_filters_registry,
    DescendsFromFilter
)
from woost.models.item import Item
from woost.models.action import Action
from woost.models.changesets import (
    ChangeSet,
    ChangeSetHasActionExpression,
    ChangeSetHasTargetExpression,
    ChangeSetHasTargetTypeExpression
)
from woost.models.publishable import Publishable, IsPublishedExpression
from woost.models.document import Document
from woost.models.usersession import get_current_user
from woost.models.expressions import OwnershipExpression


class OwnItemsFilter(UserFilter):

    id = "owned-items"

    @cached_getter
    def schema(self):
        return schema.Schema()

    @cached_getter
    def expression(self):
        return OwnershipExpression()

user_filters_registry.add(Item, OwnItemsFilter)


class IsPublishedFilter(UserFilter):

    id = "published"

    @cached_getter
    def schema(self):
        return schema.Schema()

    @cached_getter
    def expression(self):
        return IsPublishedExpression(get_current_user())

user_filters_registry.add(Publishable, IsPublishedFilter)


class TypeFilter(UserFilter):
    id = "type"
    operators = ["eq", "ne"]

    @cached_getter
    def schema(self):

        def types_search_control(parent, obj, member):
            selector = templates.new("woost.views.ContentTypePicker")
            selector.root = self.content_type
            selector.selection_mode = MULTIPLE_SELECTION
            return selector

        return schema.Schema("UserFilter", members = [
            schema.String(
                "operator",
                required = True,
                enumeration = self.operators
            ),
            schema.Collection(
                "types",
                items = schema.Reference("item_type", class_family = Item),
                search_control = types_search_control
            ),
            schema.Boolean(
                "is_inherited"
            )
        ])

    @cached_getter
    def expression(self):

        if self.operator == "eq":
            return IsInstanceExpression(Self, tuple(self.types), self.is_inherited)
        elif self.operator == "ne":
            return IsNotInstanceExpression(Self, tuple(self.types), self.is_inherited)

user_filters_registry.add(Item, TypeFilter)

class ItemSelectorFilter(schema.Reference.user_filter):

    def search_control(self, parent, obj, member):
        control = templates.new("woost.views.ItemSelector")
        control.existing_items_only = True
        return control

schema.Reference.user_filter = ItemSelectorFilter

def _collection_search_control(self, parent, obj, member):
    control = templates.new("woost.views.ItemSelector")
    control.existing_items_only = True
    return control

CollectionFilter.search_control = _collection_search_control


class ChangeSetActionFilter(UserFilter):

    @cached_getter
    def schema(self):
        return schema.Schema("UserFilter", members = [
            schema.String(
                "value",
                required = True,
                enumeration = ["create", "modify", "delete"],
                translate_value = lambda value:
                    "" if not value
                    else translations("woost %s action" % value)
            )
        ])

    @getter
    def expression(self):
        return ChangeSetHasActionExpression(
            Self,
            Action.get_instance(identifier = self.value)
        )


class ChangeSetTargetFilter(UserFilter):

    @cached_getter
    def schema(self):
        return schema.Schema("UserFilter", members = [
            schema.Reference(
                "value",
                type = Item,
                required = True
            )
        ])

    def search_control(self, parent, obj, member):
        control = templates.new("woost.views.ItemSelector")
        control.existing_items_only = True
        return control

    @getter
    def expression(self):
        return ChangeSetHasTargetExpression(Self, self.value)


class ChangeSetTargetTypeFilter(UserFilter):

    id = "target-type"

    @cached_getter
    def schema(self):
        return schema.Schema("UserFilter", members = [
            schema.Reference(
                "value",
                class_family = Item,
                required = True
            )
        ])

    @getter
    def expression(self):
        return ChangeSetHasTargetTypeExpression(Self, Constant(self.value))

user_filters_registry.add(ChangeSet, ChangeSetTargetTypeFilter)

user_filters_registry.add(Publishable, DescendsFromFilter)
user_filters_registry.set_filter_parameter(
    Publishable,
    DescendsFromFilter,
    "relation", Document.children
)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2009
"""
from cocktail.controllers.controller import Controller


class PaymentNotificationController(Controller):

    def __call__(self, **parameters):
        from woost.extensions.payments import PaymentsExtension
        gateway = PaymentsExtension.instance.payment_gateway
        payment = gateway.process_notification(parameters)
        gateway.transaction_notified(payment = payment)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from urllib import urlopen, urlencode
from simplejson import dumps, loads
from poster.encode import multipart_encode
from poster.streaminghttp import register_openers
import urllib2
register_openers()

from cocktail.translations import (
    get_language,
    translations
)
from cocktail import schema
from woost.models import Item, Language
from woost.extensions.opengraph import OpenGraphExtension

facebook_locales = {
    "Catalan": "1",
    "Czech": "2",
    "Welsh": "3",
    "Danish": "4",
    "German": "5",
    "English (Upside Down)": "51",
    "English (US)": "6",
    "Spanish": "23",
    "Spanish (Spain)": "7",
    "Finnish": "8",
    "French (France)": "9",
    "Hungarian": "30",
    "Italian": "10",
    "Japanese": "11",
    "Korean": "12",
    "Norwegian (bokmal)": "13",
    "Dutch": "14",
    "Polish": "15",
    "Portuguese (Brazil)": "16",
    "Portuguese (Portugal)": "31",
    "Romanian": "32",
    "Russian": "17",
    "Slovak": "33",
    "Slovenian": "34",
    "Swedish": "18",
    "Thai": "35",
    "Turkish": "19",
    "Simplified Chinese (China)": "20",
    "Traditional Chinese (Hong Kong)": "21",
    "Traditional Chinese (Taiwan)": "22",
    "Afrikaans": "36",
    "Bengali": "45",
    "Bulgarian": "37",
    "Croatian": "38",
    "English (UK)": "24",
    "French (Canada)": "44",
    "Greek": "39",
    "Hindi": "46",
    "Indonesian": "25",
    "Lithuanian": "40",
    "Malay": "41",
    "Punjabi": "47",
    "Serbian": "42",
    "Filipino": "26",
    "Tamil": "48",
    "Telugu": "49",
    "Malayalam": "50",
    "Vietnamese": "27",
    "Arabic": "28",
    "Hebrew": "29"
}


class FacebookPublicationTarget(Item):

    members_order = [
        "title",
        "graph_object_id",
        "administrator_id",
        "app_id",
        "app_secret",
        "auth_token",
        "languages",
        "targeting"
    ]

    visible_from_root = False

    title = schema.String(
        required = True,
        unique = True,
        indexed = True,
        normalized_index = True,
        descriptive = True,
        listed_by_default = False
    )

    graph_object_id = schema.String(
        required = True
    )

    administrator_id = schema.String(
        listed_by_default = False
    )

    app_id = schema.String(
        required = True,
        listed_by_default = False
    )

    app_secret = schema.String(
        required = True,
        listed_by_default = False,
        text_search = False
    )

    auth_token = schema.String(
        editable = False,
        text_search = False,
        translate_value = lambda value, language = None, **kwargs:
            translations(
                "FacebookPublicationTarget.auth_token-%s"
                % ("conceded" if value else "pending"),
                language,
                **kwargs
            )
    )

    languages = schema.Collection(
        items = schema.String(
            enumeration = lambda ctx: Language.codes,
            translate_value = lambda value, language = None, **kwargs:
                "" if not value else translations(value)
        )
    )

    targeting = schema.CodeBlock(
        language = "python"
    )

    def publish(self, publishable):

        if self.auth_token is None:
            raise ValueError(
                "Can't publish %s to %s: missing authorization token."
                % (publishable, self)
            )

        graph_url = "https://graph.facebook.com/%s/feed" % self.graph_object_id
        post_data = self._get_publication_parameters(publishable)
        encoded_post_data = dict(
            (k, v.encode("utf-8") if isinstance(v, unicode) else v)
            for k, v in post_data.iteritems()
        )
        response = urlopen(graph_url, urlencode(encoded_post_data))
        status = response.getcode()
        if status < 200 or status > 299:
            raise FacebookPublicationError(response.read())

    def _get_publication_parameters(self, publishable):

        og = OpenGraphExtension.instance
        og_properties = og.get_properties(publishable)

        post_data = {
            "access_token": self.auth_token,
            "name": og_properties.get("og:title") or translations(publishable),
            "link": og_properties.get("og:url")
        }

        # Disregard any default image; only show an image if the published
        # content defines one
        if "og:image" not in og_properties:
            post_data["picture"] = ""

        if self.targeting:
            context = {
                "language": get_language(),
                "og_properties": og_properties,
                "publishable": publishable,
                "locales": facebook_locales,
                "include_locales": (
                    lambda included: ",".join(
                        facebook_locales[loc_name] for loc_name in included
                    )
                ),
                "exclude_locales": (
                    lambda excluded: ",".join(
                        loc_id
                        for loc_name, loc_id in facebook_locales.iteritems()
                        if loc_name not in excluded
                    )
                )
            }
            targeting_code = self.targeting.replace("\r\n", "\n")
            exec targeting_code in context
            targeting = context.get("targeting")
            if targeting:
                post_data["targeting"] = dumps(targeting)

        return post_data

    def feed_posts(self):

        if self.auth_token is None:
            raise ValueError(
                "Can't read the posts in %s: missing authorization token."
                % self
            )

        graph_url = "https://graph.facebook.com/%s/feed/?access_token=%s" % (
            self.graph_object_id,
            self.auth_token
        )
        response = urlopen(graph_url)
        status = response.getcode()
        body = response.read()
        if status < 200 or status > 299:
            raise FacebookPublicationError(body)

        feed_data = loads(body)
        return feed_data["data"]

    def find_post(self, publishable):

        uri = self._get_publication_parameters(publishable)["link"]

        for post in self.feed_posts():
            if post.get("link") == uri:
                return post

    def publish_album(self,
        album_title,
        photos,
        album_description = None,
        photo_languages = None,
        generate_story = True):

        if self.auth_token is None:
            raise ValueError(
                "Can't publish album to %s: missing authorization token."
                % self
            )

        # Create the album
        response = urlopen(
            "https://graph.facebook.com/%s/albums" % self.graph_object_id,
            _encode_form({
                "access_token": self.auth_token,
                "name": album_title,
                "message": album_description or ""
            })
        )

        status = response.getcode()
        body = response.read()
        if status < 200 or status > 299:
            raise FacebookPublicationError(body)

        album_id = loads(body)["id"]
        photos_url = "https://graph.facebook.com/%s/photos" % album_id

        # Upload photos
        for photo in photos:

            if photo_languages is None:
                photo_languages = photo.translations.keys()
            else:
                photo_languages = list(
                    set(photo_languages) & set(photo.translations.keys())
                )

            if not photo_languages:
                photo_desc = ""
            elif len(photo_languages) == 1:
                photo_desc = translations(photo, photo_languages[0])
            else:
                photo_desc = u"\n".join(
                    u"%s: %s" % (
                        translations(lang, lang),
                        translations(photo, lang)
                    )
                    for lang in photo_languages
                )

            datagen, headers = multipart_encode({
                "access_token": self.auth_token,
                "source": open(photo.file_path),
                "message": photo_desc,
                "no_story": ("0" if generate_story else "1")
            })
            request = urllib2.Request(photos_url, datagen, headers)
            response = urllib2.urlopen(request)

            status = response.getcode()
            body = response.read()
            if status < 200 or status > 299:
                raise FacebookPublicationError(body)

        response = urlopen(
            "https://graph.facebook.com/%s/?access_token=%s" % (
                album_id,
                self.auth_token
            )
        )
        status = response.getcode()
        body = response.read()
        if status < 200 or status > 299:
            raise FacebookPublicationError(body)

        return loads(body)

def _encode_form(form):
    return urlencode(dict(
        (k, v.encode("utf-8") if isinstance(v, unicode) else v)
        for k, v in form.iteritems()
    ))


class FacebookPublicationError(Exception):

    def __init__(self, response):
        Exception.__init__(self, response)
        self.response = response




#-*- coding: utf-8 -*-
u"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			March 2009
"""

import cherrypy
from woost.controllers import BaseCMSController


class URIController(BaseCMSController):

    def __call__(self, *args, **kwargs):
        uri = self.context["publishable"].uri
        raise cherrypy.HTTPRedirect(uri)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
from __future__ import with_statement
import cherrypy
from cocktail.modeling import cached_getter
from cocktail import schema
from cocktail.persistence import datastore
from cocktail.controllers import FormControllerMixin
from woost.models import changeset_context, get_current_user
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController
from woost.extensions.vimeo.video import VimeoVideo


class SyncVimeoController(FormControllerMixin, BaseBackOfficeController):

    view_class = "woost.extensions.vimeo.VimeoSynchronizationView"

    def __call__(self, *args, **kwargs):

        if "cancel" in kwargs:
            raise cherrypy.HTTPRedirect(self.contextual_uri())

        return BaseBackOfficeController.__call__(self, *args, **kwargs)

    @cached_getter
    def form_model(self):
        from woost.extensions.vimeo import VimeoExtension
        return schema.Schema("VimeoSynchronizationForm", members = [
            schema.String("vimeo_user_name",
                required = True,
                text_search = False,
                default = schema.DynamicDefault(
                    lambda: VimeoExtension.instance.default_vimeo_user_name
                )
            )
        ])

    def submit(self):

        # Read form data
        FormControllerMixin.submit(self)

        from woost.extensions.vimeo import VimeoExtension
        extension = VimeoExtension.instance
        user = get_current_user()

        with changeset_context(author = user) as changeset:
            extension.synchronize(
                self.form_instance["vimeo_user_name"],
                restricted = True
            )

        # Report changed videos
        created = set()
        modified = set()
        deleted = set()

        for change in changeset.changes.itervalues():

            if isinstance(change.target, VimeoVideo):
                action_id = change.action.identifier

                if action_id == "create":
                    created.add(change.target)
                elif action_id == "modify":
                    modified.add(change.target)
                elif action_id == "delete":
                    deleted.add(change.target)

        datastore.commit()

        self.output["created_videos"] = created
        self.output["modified_videos"] = modified
        self.output["deleted_videos"] = deleted




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2009
"""
from __future__ import with_statement
from contextlib import contextmanager
from cocktail.translations import translations
from cocktail.tests.seleniumtester import selenium_test, browser
from woost.models import Item
from woost.tests.test_selenium import admin_login

@contextmanager
def override(obj, **kwargs):

    undefined = object()

    prev_values = {}

    try:
        for key, value in kwargs.iteritems():
            prev_values[key] = getattr(obj, key, undefined)
            setattr(obj, key, value)

        yield

    finally:
        for key, value in prev_values.iteritems():
            if value is undefined:
                delattr(obj, key)
            else:
                setattr(obj, key, value)


class SearchTestCase(object):

    @selenium_test
    def test_unfold_expanded_search(self):
        browser.open("/en/cms/content/?content_view=flat")
        admin_login()
        browser.click("css=.advanced_search")
        browser.wait_for_page_to_load(10000)
        assert browser.is_element_present("css=.filters")
        assert browser.is_element_present("css=.filters .new_filter_selector")

    @selenium_test
    def test_add_filters_from_selector(self):

        from cocktail.schema.expressions import Self
        from cocktail.persistence import Query

        results = Item.select([
            Item.id.greater(15),
            Self.search("@localhost", languages = ["en"])
        ])
        results_count = len(results)

        browser.open(
            "/en/cms/content/?content_view=flat&search_expanded=true"
            "&page_size=%d" % (results_count + 1)
        )
        admin_login()

        # Add filters
        for i, filter_id in enumerate(("member-id", "global_search")):
            browser.fire_event("css=.new_filter_selector", "click")
            browser.fire_event("css=.new_filter-%s" % filter_id, "click")

        assert browser.jquery_count(".filters .filter_list .filter_entry") == 2

        # Set values on filters
        browser.type("filter_operator0", "gt")
        browser.type("filter_value0", "15")
        browser.type("filter_value1", "@localhost")
        browser.click("css=.filters .search_button")
        browser.wait_for_page_to_load(10000)

        # Test the returned content
        rows_count = browser.jquery_count(".collection_display .item_row")
        assert rows_count == results_count

        # Supplied values must be preserved between page loads
        assert browser.get_selected_value("filter_operator0") == "gt"
        assert browser.get_value("filter_value0") == "15"
        assert browser.get_value("filter_value1") == "@localhost"

    @selenium_test
    def test_multiple_options(self):

        from woost.models import User, Role
        from cocktail.controllers.userfilter import MultipleChoiceFilter

        administrators = Role.require_instance(
            qname = "woost.administrators"
        )

        anonymous = Role.require_instance(
            qname = "woost.anonymous"
        )

        with override(User.roles, user_filter = MultipleChoiceFilter):

            browser.open("/en/cms/content/?content_view=flat&type=woost.models.user.User&search_expanded=true")
            admin_login()

            # Select the filter
            browser.fire_event("css=.new_filter_selector", "click")
            browser.fire_event("css=.new_filter-member-roles", "click")

            # Select options
            browser.fire_event("css=.filters .values_field .select", "click")
            assert browser.is_visible("css=.modal_selector_dialog")

            for role in (administrators, anonymous):
                browser.click(
                    "css=.modal_selector_dialog input[value=%d]"
                    % role.id
                )

            browser.fire_event("css=.modal_selector_dialog .accept", "click")

            for role in (administrators, anonymous):
                assert browser.is_element_present(
                    "css=.filters .values_field input[value=%d]"
                    % role.id
                )

            browser.click("css=.filters .search_button")
            browser.wait_for_page_to_load(10000)

            # Validate results
            results_count = len(administrators.users) + len(anonymous.users)
            rows_count = browser.jquery_count(".collection_display .item_row")
            assert rows_count == results_count

            # Make sure the search control preserves its state
            for role in (administrators, anonymous):
                assert browser.is_element_present(
                    "css=.filters .values_field input[value=%d]"
                    % role.id
                )

            browser.fire_event("css=.filters .values_field .select", "click")

            for role in (administrators, anonymous):
                assert browser.is_checked(
                    "css=.modal_selector_dialog input[value=%d]"
                    % role.id
                )

    @selenium_test
    def test_item_selector(self):

        from woost.models import Publishable, Template
        template = list(Template.select())[0]

        browser.open("/en/cms/content/?content_view=flat&type=woost.models.publishable.Publishable&search_expanded=true")
        admin_login()

        browser.fire_event("css=.new_filter_selector", "click")
        browser.fire_event("css=.new_filter-member-template", "click")
        assert not browser.is_element_present("css=.filters .value_field .new")
        assert not browser.is_element_present("css=.filters .value_field .edit")
        assert not browser.is_element_present("css=.filters .value_field .delete")
        assert not browser.is_element_present("css=.filters .value_field .unlink")
        browser.click("css=.filters .value_field .select")

        browser.select_frame("dom=selenium.browserbot.getCurrentWindow().frames[0]")
        browser.wait_for_element_present("css=.collection_display", 10000)
        assert not browser.is_element_present("css=.new_action")
        browser.click("css=.collection_display #%d" % template.id)
        browser.click("css=.select_action")

        browser.select_frame("relative=parent")
        browser.wait_for_element_present("css=.collection_display", 10000)

        assert translations(template, "en") \
            in browser.jquery_html(".filters .value_field")

        assert browser.get_value("filter_value0") == str(template.id)

        browser.click("css=.filters .search_button")
        browser.wait_for_page_to_load(10000)

        rows_count = browser.jquery_count(".collection_display .item_row")
        assert rows_count == len(template.items)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail import schema
from woost.models import File, Publishable
from woost.models.rendering import ImageFactoryMember
from woost.extensions.blocks.block import Block


class BannerBlock(Block):

    instantiable = True
    tag = "a"
    view_class = "woost.extensions.blocks.Banner"

    members_order = [
        "target",
        "image",
        "image_factory",
        "text",
        "label_displayed"
    ]

    image = schema.Reference(
        type = File,
        integral = True,
        related_end = schema.Collection(),
        member_group = "content"
    )

    image_factory = ImageFactoryMember(
        member_group = "content"
    )

    target = schema.Reference(
        type = Publishable,
        required = True,
        related_end = schema.Collection(cascade_delete = True),
        member_group = "content"
    )

    text = schema.String(
        translated = True,
        member_group = "content"
    )

    label_displayed = schema.Boolean(
        default = False,
        required = True,
        member_group = "content"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view.image = self.image
        view.image_factory = self.image_factory
        view.target = self.target
        view.text = self.text
        view.label_displayed = self.label_displayed




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
from cocktail.html import Element, templates
from woost.models import Item

ContentTypeSelector = \
    templates.get_class("woost.views.ContentTypeSelector")


class ContentTypePath(Element):

    tag = "ul"
    value = None
    root = Item

    Entry = Element
    Selector = ContentTypeSelector

    def _ready(self):

        inheritance_line = []

        for content_type in self.value.ascend_inheritance(True):
            inheritance_line.insert(0, content_type)
            if content_type is self.root:
                break

        for content_type in inheritance_line:
            entry = self.create_entry(content_type)
            self.append(entry)

        self._last_entry = entry
        Element._ready(self)

    def _content_ready(self):

        Element._content_ready(self)

        for derived_content_type in self.value.derived_schemas():
            break
        else:
            self._last_entry.selector.tree.visible = False
            self._last_entry.add_class("leaf")

    def create_entry(self, content_type):
        entry = self.Entry()
        entry.tag = "li"
        entry.selector = self.create_selector(content_type)
        entry.append(entry.selector)
        return entry

    def create_selector(self, content_type):
        selector = self.Selector()
        selector.root = content_type
        selector.value = content_type

        if content_type is self.value:
            selector.add_class("selected")

        return selector




#-*- coding: utf-8 -*-
u"""

@author:        Jordi Fernández
@contact:       jordi.fernandez@whads.com
@organization:  Whads/Accent SL
@since:         March 2010
"""
from cocktail import schema

SubscriptionForm = schema.Schema("SubscriptionForm", members = [
    schema.String("email",
        required = True,
        format = r"([\w\-\.]+@(\w[\w\-]+\.)+[\w\-]+)"
    ),
    schema.String("name")
])

SubscriptionForm.members_order = "name", "email"



#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			December 2009
"""
from cocktail.translations import translations
from woost.translations.strings import content_permission_translation_factory

# UI
#------------------------------------------------------------------------------
translations.define("Action export_static",
    ca = u"Exportar contingut estàtic",
    es = u"Exportar contenido estático",
    en = u"Export static content"
)

translations.define("woost.extensions.staticsite any destination",
    ca = u"Tots els destins",
    es = u"Todos los destinos",
    en = u"All destinations"
)

translations.define("woost.extensions.staticsite Export button",
    ca = u"Exportar",
    es = u"Exportar",
    en = u"Export"
)

translations.define("woost.extensions.staticsite export done",
    ca = u"Exportació completada",
    es = u"Exportación completada",
    en = u"Content exported"
)

translations.define("woost.extensions.staticsite download",
    ca = u"Pots descarregar el contingut <a href='%s'>aquí</a>",
    es = u"Puedes descargar el contenido <a href='%s'>aquí</a>",
    en = u"Download the content <a href='%s'>here</a>"
)

translations.define("woost.extensions.staticsite.export_status-ignored",
    ca = u"Ignorat (no exportable)",
    es = u"Ignorado (no exportable)",
    en = u"Ignored (not exportable))"
)

translations.define("woost.extensions.staticsite.export_status-not_modified",
    ca = u"Ignorat (no modificat)",
    es = u"Ignorado (no modificado)",
    en = u"Ignored (not modified)"
)

translations.define("woost.extensions.staticsite.export_status-exported",
    ca = u"Exportat",
    es = u"Exportado",
    en = u"Exported"
)

translations.define("woost.extensions.staticsite.export_status-failed",
    ca = u"Fallit",
    es = u"Fallido",
    en = u"Failed"
)

translations.define("woost.extensions.staticsite export info",
    ca = u"S'exportarà els següents elements:",
    es = u"Se exportará los siguientes elementos:",
    en = u"The following items will be exported:"
)

# Export form
#------------------------------------------------------------------------------
translations.define("ExportStaticSite.update_only",
    ca = u"Exportar únicament el contingut modificat",
    es = u"Exportar únicamente el contenido modificado",
    en = u"Only export modified content"
)

translations.define("ExportStaticSite.exporter",
    ca = u"Destí",
    es = u"Destino",
    en = u"Destination"
)

translations.define("ExportStaticSite.snapshoter",
    ca = u"Exportador",
    es = u"Exportador",
    en = u"Exporter"
)

translations.define("ExportStaticSite.follow_links",
    ca = u"Seguir els enllaços",
    es = u"Seguir los enlaces",
    en = u"Follow links"
)

# StaticSiteExtension
#------------------------------------------------------------------------------
translations.define("StaticSiteExtension.snapshoters",
    ca = u"Exportadors",
    es = u"Exportadores",
    en = u"Exporters"
)

translations.define("StaticSiteExtension.destinations",
    ca = u"Destins",
    es = u"Destinos",
    en = u"Destinations"
)

# StaticSiteSnapShoter
#------------------------------------------------------------------------------
translations.define("StaticSiteSnapShoter",
    ca = u"Exportador de contingut estàtic",
    es = u"Exportador de contenido estático",
    en = u"Static content exporter"
)

translations.define("StaticSiteSnapShoter-plural",
    ca = u"Exportadors de contingut estàtic",
    es = u"Exportadores de contenido estático",
    en = u"Static content exporters"
)

# WgetSnapShoter
#------------------------------------------------------------------------------
translations.define("WgetSnapShoter",
    ca = u"Exportador Wget",
    es = u"Exportador Wget",
    en = u"Wget exporter"
)

translations.define("WgetSnapShoter-plural",
    ca = u"Exportadors Wget",
    es = u"Exportadores Wget",
    en = u"Wget exporters"
)

translations.define("WgetSnapShoter.file_names_mode",
    ca = u"Mode dels noms de fitxers",
    es = u"Modo de los nombres de ficheros",
    en = u"File names mode"
)

translations.define("woost.extensions.staticsite.staticsitesnapshoter.WgetSnapShoter.file_names_mode unix",
    ca = u"Unix",
    es = u"Unix",
    en = u"Unix"
)

translations.define("woost.extensions.staticsite.staticsitesnapshoter.WgetSnapShoter.file_names_mode windows",
    ca = u"Windows",
    es = u"Windows",
    en = u"Windows"
)

# StaticSiteDestination
#------------------------------------------------------------------------------
translations.define("StaticSiteDestination",
    ca = u"Destí de contingut estàtic",
    es = u"Destino de contenido estático",
    en = u"Static content destination"
)

translations.define("StaticSiteDestination-plural",
    ca = u"Destins de contingut estàtic",
    es = u"Destinos de contenido estático",
    en = u"Static content destinations"
)

translations.define("StaticSiteDestination.destination_permissions",
    ca = u"Permisos",
    es = u"Permisos",
    en = u"Permissions"
)

translations.define("StaticSiteDestination.encoding",
    ca = u"Codificació dels fitxers",
    es = u"Codificación de los ficheros",
    en = u"File encoding"
)

# FolderDestination
#------------------------------------------------------------------------------
translations.define("FolderDestination",
    ca = u"Carpeta local",
    es = u"Carpeta local",
    en = u"Local folder"
)

translations.define("FolderDestination-plural",
    ca = u"Carpeta local",
    es = u"Carpeta local",
    en = u"Local folder"
)

translations.define("FolderDestination.target_folder",
    ca = u"Carpeta de destí",
    es = u"Carpeta de destino",
    en = u"Target folder"
)

# FTPDestination
#------------------------------------------------------------------------------
translations.define("FTPDestination",
    ca = u"Servidor FTP",
    es = u"Servidor FTP",
    en = u"FTP server"
)

translations.define("FTPDestination-plural",
    ca = u"Servidor FTP",
    es = u"Servidor FTP",
    en = u"FTP server"
)

translations.define("FTPDestination.ftp_host",
    ca = u"Adreça del servidor FTP",
    es = u"Dirección del servidor FTP",
    en = u"FTP host"
)

translations.define("FTPDestination.ftp_user",
    ca = u"Usuari FTP",
    es = u"Usuario FTP",
    en = u"FTP user"
)

translations.define("FTPDestination.ftp_password",
    ca = u"Contrasenya FTP",
    es = u"Contraseña FTP",
    en = u"FTP password"
)

translations.define("FTPDestination.ftp_path",
    ca = u"Carpeta de destí",
    es = u"Carpeta de destino",
    en = u"Target folder"
)

# ZipDestination
#------------------------------------------------------------------------------
translations.define("ZipDestination",
    ca = u"Fitxer ZIP",
    es = u"Fichero ZIP",
    en = u"ZIP file"
)

translations.define("ZipDestination-plural",
    ca = u"Fitxer ZIP",
    es = u"Fichero ZIP",
    en = u"ZIP file"
)

translations.define(
    "woost.extensions.staticsite.staticsiteexporter."
    "ZipDestination-instance",
    ca = u"Fitxer ZIP",
    es = u"Fichero ZIP",
    en = u"ZIP file"
)

# ExportationPermission
#------------------------------------------------------------------------------
translations.define("ExportationPermission",
    ca = u"Permís d'exportació",
    es = u"Permiso de exportación",
    en = u"Exportation permission"
)

translations.define("ExportationPermission-plural",
    ca = u"Permisos d'exportacions",
    es = u"Permisos de exportaciones",
    en = u"Exportation permissions"
)

translations.define("ExportationPermission.destination",
    ca = u"Destí",
    es = u"Destino",
    en = u"Exportation"
)

translations.define(
    "woost.extensions.workflow.exportationpermission."
    "ExportationPermission-instance",
    ca = content_permission_translation_factory("ca",
        lambda permission, subject, **kwargs:
            "%s %s" % (
                translations(permission.destination, "ca", **kwargs),
                subject
            )
            if permission.destination
            else "canviar l'estat " + ca_possessive(subject),
    ),
    es = content_permission_translation_factory("es",
        lambda permission, subject, **kwargs:
            "%s %s" % (
                translations(permission.destination, "es", **kwargs),
                subject
            )
            if permission.destination
            else "cambiar el estado de " + subject,
    ),
    en = content_permission_translation_factory("en",
        lambda permission, subject, **kwargs:
            "%s %s" % (
                translations(permission.destination, "en", **kwargs),
                subject
            )
            if permission.destination
            else "change the state of " + subject
    )
)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
import cherrypy
from cocktail import schema
from cocktail.controllers import get_parameter
from woost.extensions.shop.shopcontroller import ShopController
from woost.extensions.shop.product import Product
from woost.extensions.shop.basket import Basket


class BasketController(ShopController):

    @cherrypy.expose
    def update(self, **kwargs):

        product = get_parameter(schema.Reference("product", type = Product))
        quantity = get_parameter(schema.Integer("quantity", min = 0))

        if product is not None and quantity is not None:
            shop_order = Basket.get()
            shop_order.set_product_quantity(product, quantity)
            Basket.store()

        self.parent_redirect()

    def parent_redirect(self):
        parts = cherrypy.request.path_info.strip("/").split("/")
        parts.pop(-1)
        path = "/" + "/".join(parts)
        raise cherrypy.HTTPRedirect(path)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from woost.models import Document, File, User
from woost.controllers.backoffice.useractions import UserAction
from woost.extensions.facebookpublication.facebookpublicationpermission \
    import FacebookPublicationPermission


class BaseFBPublishUserAction(UserAction):

    included = frozenset([
        ("content", "toolbar_extra"),
        ("collection", "toolbar_extra", "integral"),
        "item_buttons_extra"
    ])

    excluded = frozenset([
        "selector",
        "new_item",
        "calendar_content_view",
        "workflow_graph_content_view",
        "changelog"
    ])

    max = None

    def is_permitted(self, user, target):
        return user.has_permission(FacebookPublicationPermission, target = target)


class FBPublishUserAction(BaseFBPublishUserAction):
    content_type = Document

FBPublishUserAction("fbpublish").register()


class FBPublishAlbumsUserAction(BaseFBPublishUserAction):

    content_type = File

    def is_available(self, context, target):

        if not BaseFBPublishUserAction.is_available(self, context, target):
            return False

        return isinstance(target, type) or target.resource_type == "image"

FBPublishAlbumsUserAction("fbpublish_albums").register()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
from webconsole import WebConsoleController
from cocktail.events import event_handler
from cocktail.iteration import first, last
from cocktail.translations import translations, get_language
from cocktail.persistence import PersistentObject
from cocktail.controllers import request_property
from woost.controllers.documentcontroller import DocumentController
from woost.models import Site, get_current_user
from woost.extensions.webconsole.webconsolepermission \
    import WebConsolePermission


class CMSWebConsole(DocumentController, WebConsoleController):

    @event_handler
    def handle_traversed(self, event):
        get_current_user().require_permission(WebConsolePermission)

    submit = WebConsoleController.submit
    submitted = WebConsoleController.submitted

    @request_property
    def session(self):

        session = WebConsoleController.session(self)

        session.context.update(
            site = Site.main,
            user = get_current_user(),
            language = get_language(),
            translations = translations,
            first = first,
            last = last
        )

        for model in PersistentObject.schema_tree():
            session.context[model.name] = model

        return session




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			May 2009
"""
from __future__ import with_statement
from base64 import b64encode
from cocktail.modeling import cached_getter, InstrumentedSet
from cocktail.controllers import context
from cocktail.controllers.webservices import (
    PersistentClassWebService,
    excluded_member
)
from woost.models import (
    changeset_context,
    Item,
    User,
    Site,
    Language,
    get_current_user,
    restricted_modification_context,
    PermissionExpression,
    ReadPermission,
    DeletePermission,
    ReadMemberPermission,
    ReadTranslationPermission
)
from woost.controllers.basecmscontroller import BaseCMSController


class ItemWebService(PersistentClassWebService):
    root_type = Item

    def _init_user_collection(self, user_collection):
        PersistentClassWebService._init_user_collection(self, user_collection)
        user_collection.add_base_filter(
            PermissionExpression(get_current_user(), ReadPermission)
        )

    @cached_getter
    def languages(self):
        return Language.codes

    class JSONEncoder(PersistentClassWebService.JSONEncoder):

        def _member_permission(self, member):
            permission = self._member_permissions.get(member)
            if permission is None:
                permission = self.user.has_permission(
                    ReadMemberPermission,
                    member = member
                )
                self._member_permissions[member] = permission
            return permission

        def _language_permission(self, language):
            permission = self._language_permissions.get(language)
            if permission is None:
                permission = self.user.has_permission(
                    ReadTranslationPermission,
                    language = language
                )
                self._language_permissions[language] = permission
            return permission

        def get_member_value(self, obj, member, language = None):

            # Exclude the 'changes' member
            if member is Item.changes:
                value = excluded_member

            # Exclude restricted members
            elif not (
                self._language_permission(language)
                and self._member_permission(member)
            ):
                value = excluded_member

            # Special case for user passwords
            elif member is User.password:
                value = obj.password
                if value:
                    value = b64encode(value)
            else:
                value = PersistentClassWebService.JSONEncoder.get_member_value(
                    self,
                    obj,
                    member,
                    language
                )

            return value

    @cached_getter
    def json_encoder(self):
        encoder = PersistentClassWebService.json_encoder(self)
        encoder.user = get_current_user()
        encoder._language_permissions = {}
        encoder._member_permissions = {}
        return encoder

    def _init_new_instance(self, instance):
        with restricted_modification_context(instance):
            PersistentClassWebService._init_new_instance(self, instance)

    def _store_new_instance(self, instance):
        if instance.is_draft:
            instance.insert()
        else:
            with changeset_context(get_current_user()):
                instance.insert()

    def _update_instance(self, instance):
        with restricted_modification_context(instance):
            if instance.is_draft:
                PersistentClassWebService._update_instance(self, instance)
            else:
                with changeset_context(get_current_user()):
                    PersistentClassWebService._update_instance(self, instance)

    def _delete_instances(self, query):

        user = get_current_user()

        class ValidatingDeletedSet(InstrumentedSet):
            def item_added(self, item):
                user.require_permission(DeletePermission, target = item)

        deleted_set = ValidatingDeletedSet()

        with changeset_context(user):
            for item in list(query):
                item.delete(deleted_set)


class WebServicesController(BaseCMSController):
    data = ItemWebService




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			September 2010
"""
from cocktail.translations import translations
from cocktail.translations.strings import member_identifier

# UI
#------------------------------------------------------------------------------
translations.define("Action send_email",
    ca = u"Enviar",
    es = u"Enviar",
    en = u"Send"
)

translations.define("Action create_mailing",
    ca = u"Enviar per correu",
    es = u"Enviar por correo",
    en = u"Send by e-mail"
)

translations.define("woost.extensions.mailer users",
    ca = u"usuaris",
    es = u"usuarios",
    en = u"usuarios"
)

# Mailing
#------------------------------------------------------------------------------
translations.define("Mailing",
    ca = u"Enviament",
    es = u"Envio",
    en = u"Mailing"
)

translations.define("Mailing-plural",
    ca = u"Enviaments",
    es = u"Envios",
    en = u"Mailings"
)

translations.define("Mailing.content",
    ca = u"Contingut",
    es = u"Contenido",
    en = u"Content"
)

translations.define("Mailing.document",
    ca = u"Document",
    es = u"Documento",
    en = u"Document"
)

translations.define("Mailing.sender",
    ca = u"Remitent",
    es = u"Remitente",
    en = u"Sender"
)

translations.define("Mailing.subject",
    ca = u"Assumpte",
    es = u"Asunto",
    en = u"Subject"
)

translations.define("Mailing.roles",
    ca = u"Rols",
    es = u"Roles",
    en = u"Roles"
)

translations.define("Mailing.language",
    ca = u"Idioma",
    es = u"Idioma",
    en = u"Language"
)

translations.define("Mailing.per_user_customizable",
    ca = u"Personalitzable per usuari",
    es = u"Personalizable por usuario",
    en = u"Per user customizable"
)

translations.define("Mailing.status",
    ca = u"Estat",
    es = u"Estado",
    en = u"Status"
)

translations.define("woost.extensions.mailer.mailing.Mailing.status 1",
    ca = u"Iniciat",
    es = u"Iniciado",
    en = u"Started"
)

translations.define("woost.extensions.mailer.mailing.Mailing.status 2",
    ca = u"Finalitzat",
    es = u"Finalizado",
    en = u"Finished"
)

# SendEmailView
#------------------------------------------------------------------------------
translations.define("woost.extensions.mailer.SendEmailView continue",
    ca = u"Continuar",
    es = u"Continuar",
    en = u"Continue"
)

translations.define("woost.extensions.mailer.SendEmailView send",
    ca = u"Enviar",
    es = u"Enviar",
    en = u"Send"
)

translations.define("woost.extensions.mailer.SendEmailView confirmation text",
    ca = lambda mailing:
        u"S'enviarà el document <strong>%s</strong> en <strong>%s</strong> "
        u"als següents usuaris:"
        % (translations(mailing.document), translations(mailing.language)),
    es = lambda mailing:
        u"Se enviará el documento <strong>%s</strong> en "
        u"</strong>%s</strong> a los siguientes usuarios:"
        % (translations(mailing.document), translations(mailing.language)),
    en = lambda mailing:
        u"You are about to send the document <strong>%s</strong> "
        u"in <strong>%s</strong> to the following users:"
        % (translations(mailing.document), translations(mailing.language))
)

translations.define("woost.extensions.mailer.SendEmailView total",
    ca = u"Total:",
    es = u"Total:",
    en = u"Total:"
)

translations.define("woost.extensions.mailer.SendEmailView accessibility warning",
    ca = u"""Tingues en compte que en aquests moments el document que
estàs enviant no és accessible pels usuaris anònims.""",
    es = u"""Ten en cuenta que en estos momentos el documento que
estás enviando no és accesible para los usuarios anónimos.""",
    en = u"""Note that at this moment the document you are sending
is not accessible for anonymous users."""
)

translations.define("woost.extensions.mailer.SendEmailView doing email delivery",
    ca = u"Realitzant enviament",
    es = u"Realizando envio",
    en = u"Doing email delivery"
)

translations.define("woost.extensions.mailer.SendEmailView zero per cent",
    ca = u"0%",
    es = u"0%",
    en = u"0%"
)

translations.define("woost.extensions.mailer.SendEmailView email delivery finished",
    ca = u"Enviament finalitzat",
    es = u"Envio finalizado",
    en = u"Mail delivery finished"
)

translations.define("woost.extensions.mailer.SendEmailView error email delivery",
    ca = u"No s'ha pogut enviar alguns correus. Posa't en contacte amb l'administrador.",
    es = u"No se ha podido enviar algunos correos. Ponte en contacto con el administrador.",
    en = u"Unable to send some emails. Please, contact your administrator."
)

translations.define("woost.extensions.mailer.SendEmailView mailer summary",
    ca = u"S'ha enviat <strong>{0}</strong> correus de <strong>{1}</strong>.",
    es = u"Se han enviado <strong>{0}</strong> correos de <strong>{1}</strong>.",
    en = u"<strong>{0}</strong> of <strong>{1}</strong> emails have been sent."
)

translations.define("woost.extensions.mailer.SendEmailView test explanation",
    ca = u"""Degut a que molts clients de correu electrònic mostren el disseny
del correu de forma diferent, recomanem enviar alguna prova abans de realitzar
l'enviament definitiu.""",
    es = u"""Debido a que muchos clientes de correo electrónico muestran el
diseño del correo de forma diferente, recomendamos enviar alguna prueba antes
de realitzar el envio definitivo.""",
    en = u"""Because most email clients will display your email design
differently, we recommend sending yourself a few tests before scheduling the
final campaign."""
)

translations.define("woost.extensions.mailer.SendEmailView test email",
    ca = u"Correu electrònic",
    es = u"Correo electrónico",
    en = u"E-mail address"
)

translations.define("woost.extensions.mailer.SendEmailView test",
    ca = u"Enviar el correu de prova",
    es = u"Enviar el correo de prueba",
    en = u"Send the test email"
)

# Template
#------------------------------------------------------------------------------
translations.define("Template.per_user_customizable",
    ca = u"Personalitzable per usuari",
    es = u"Personalizable por usuario",
    en = u"Per user customizable"
)

# SendEmailPermission
#------------------------------------------------------------------------------
translations.define("SendEmailPermission",
    ca = u"Permís d'enviament de correu electrònic",
    es = u"Permiso de envio de correo electrónico",
    en = u"Permission to send email"
)

translations.define("SendEmailPermission-plural",
    ca = u"Permisos d'enviament de correu electrònic",
    es = u"Permisos de envio de correo electrónico",
    en = u"Permissions to send email"
)

translations.define("SendEmailPermission.roles",
    ca = u"Rols",
    es = u"Roles",
    en = u"Roles"
)

translations.define("SendEmailPermission.roles-explanation",
    ca = u"Limita el permís en funció dels destinataris de l'enviament. "
         u"Deixar en blanc per permetre o prohibir l'enviament a tothom.",
    es = u"Limita el permiso en función de los destinatarios del envío. "
         u"Dejar en blanco para permitir o prohibir el envío sin importar "
         u"los destinatarios seleccionados.",
    en = u"Limits the groups of receivers that the user can or can't send "
         u"email to. Leave blank to allow / disallow sending e-mail "
         u"regardless of the selected receivers."
)

# Exceptions
#------------------------------------------------------------------------------
translations.define("woost.extensions.mailer.mailing.DocumentTemplateRequiredError-instance",
    ca = lambda instance:
        u"El camp <em>%s</em> ha de tindre plantilla"
        % member_identifier(instance),
    es = lambda instance:
        u"El campo <em>%s</em> debe tener plantilla"
        % member_identifier(instance),
    en = lambda instance:
        u"The <em>%s</em> field must have a template"
        % member_identifier(instance)
)

translations.define("woost.extensions.mailer.mailing.LanguageValueError-instance",
    ca = lambda instance:
        u"El camp <em>%s</em> no és un dels idiomes del document"
        % member_identifier(instance),
    es = lambda instance:
        u"El campo <em>%s</em> no es uno de los idiomas del documento"
        % member_identifier(instance),
    en = lambda instance:
        u"The <em>%s</em> field isn't one of the document languages"
        % member_identifier(instance)
)

translations.define("woost.extensions.mailer.mailing.PerUserCustomizableValueError-instance",
    ca = lambda instance:
        u"El camp <em>%s</em> no és vàlid pel document seleccionat"
        % member_identifier(instance),
    es = lambda instance:
        u"El campo <em>%s</em> no es válido para el documento seleccionado"
        % member_identifier(instance),
    en = lambda instance:
        u"The <em>%s</em> is not valid for the selected document"
        % member_identifier(instance)
)

translations.define(
    "woost.extensions.mailer.mailing.RunningMailingError-instance",
    ca = u"No es pot eliminar un enviament que està en execució.",
    es = u"No se puede eliminar un envio que está en ejecución.",
    en = u"Can't delete a running mailing."
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import os
from datetime import datetime
from cocktail import schema
from cocktail.translations import get_language
from cocktail.controllers import Location
from cocktail.persistence import datastore, PersistentMapping
from woost.models.item import Item


class CachingPolicy(Item):

    # TODO: combine per type filtering with a per item condition

    visible_from_root = False
    edit_form = "woost.views.CachingPolicyForm"

    groups_order = ["cache"]
    members_order = [
        "description",
        "important",
        "cache_enabled",
        "server_side_cache",
        "cache_expiration",
        "last_update_expression",
        "cache_key_expression",
        "condition"
    ]

    description = schema.String(
        descriptive = True,
        translated = True,
        listed_by_default = False
    )

    important = schema.Boolean(
        required = True,
        default = False
    )

    cache_enabled = schema.Boolean(
        required = True,
        default = True
    )

    server_side_cache = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False
    )

    cache_expiration = schema.Integer(
        min = 1,
        listed_by_default = False
    )

    condition = schema.CodeBlock(
        language = "python"
    )

    cache_key_expression = schema.CodeBlock(
        language = "python"
    )

    last_update_expression = schema.CodeBlock(
        language = "python"
    )

    def applies_to(self, publishable, **context):
        expression = self.condition
        if expression:
            expression = expression.replace("\r", "")
            context["publishable"] = publishable
            exec expression in context
            return context.get("applies", False)

        return True

    def get_content_cache_key(self, publishable, **context):

        cache_key = (str(Location.get_current(relative = False)),)
        key_qualifier = None
        expression = self.cache_key_expression

        if expression:
            expression = expression.replace("\r", "")
            context["publishable"] = publishable
            exec expression in context
            key_qualifier = context.get("cache_key")
        else:
            request = context.get("request")
            if request:
                key_qualifier = tuple(request.params.items())

        if key_qualifier:
            cache_key = cache_key + (key_qualifier,)

        return cache_key

    def get_content_last_update(self, publishable, **context):

        context["publishable"] = publishable
        context["latest"] = latest

        # Per model cache invalidation
        cache_expiration = datastore.root.get("woost.cache_expiration")
        dates = []

        if cache_expiration:
            for cls in publishable.__class__.__mro__:
                if cls is Item:
                    break
                dates.append(cache_expiration.get(cls.full_name))

        # Custom expression
        expression = self.last_update_expression
        if expression:
            expression = expression.replace("\r", "")
            exec expression in context
            dates.append(context.get("last_update"))

        # By default, only check the item's own last update date
        else:
            dates.append(publishable.last_update_time)

        return normalize_invalidation_date(dates)

# Utility functions for last update expressions
#------------------------------------------------------------------------------

def normalize_invalidation_date(value):

    if isinstance(value, Item):
        value = value.last_update_time

    elif hasattr(value, "__iter__"):
        max_date = None
        for item in value:
            date = normalize_invalidation_date(item)
            if date is None:
                continue
            if max_date is None or date > max_date:
                max_date = date

        value = max_date

    return value

def expire_cache(cls = None):

    if cls is None:
        from woost.models.publishable import Publishable as cls

    cache_expiration = datastore.root.get("woost.cache_expiration")

    if cache_expiration is None:
        cache_expiration = PersistentMapping()
        datastore.root["woost.cache_expiration"] = cache_expiration

    cache_expiration[cls.full_name] = datetime.now()

def latest(selectable, *args, **kwargs):

    if not args and not kwargs \
    and hasattr(selectable, "get_last_instance_change"):
        return selectable.get_last_instance_change()

    return (
        selectable.select(
            order = Item.last_update_time.negative(),
            range = (0, 1)
        )
        .select(*args, **kwargs)
    )

def menu_items(publishable):
    items = []

    while publishable is not None:
        if hasattr(publishable, "children"):
            items.extend(publishable.children)
        if publishable.parent is None:
            items.append(publishable)
        publishable = publishable.parent

    return items

def file_date(publishable):
    try:
        ts = os.stat(publishable.file_path).st_mtime
    except IOError, OSError:
        return None

    return datetime.fromtimestamp(ts)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
from cocktail.html import Element
from cocktail.translations import translations
from cocktail.controllers import context
from woost.views.itemlabel import ItemLabel


class ContentLink(ItemLabel):

    referer = None
    icon_visible = False

    def __init__(self, item = None, referer = None, **kwargs):
        Element.__init__(self, **kwargs)
        self.item = item
        self.referer = referer

    def _ready(self):

        ItemLabel._ready(self)

        if self.item:
            self.tag = "a"
            self["href"] = context["cms"].contextual_uri(
                "content", self.item.id, "show_detail"
            )
        else:
            self.append(u"-")

    def get_label(self):
        return translations(self.item, referer = self.referer)




#-*- coding: utf-8 -*-
u"""

@var extensions: A dictionary mapping extensions to MIME types.
@var extensions: dict(str, str)

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
import os
import hashlib
from mimetypes import guess_type
from shutil import copy
from cocktail.events import event_handler
from cocktail.memoryutils import format_bytes
from cocktail import schema
from cocktail.persistence import datastore
from woost import app
from woost.models.publishable import Publishable
from woost.models.controller import Controller
from woost.models.language import Language


class File(Publishable):

    instantiable = True

    edit_view = "woost.views.FileFieldsView"
    edit_node_class = \
        "woost.controllers.backoffice.fileeditnode.FileEditNode"

    default_mime_type = None

    default_encoding = None

    default_controller = schema.DynamicDefault(
        lambda: Controller.get_instance(qname = "woost.file_controller")
    )

    members_order = [
        "title",
        "file_name",
        "file_size",
        "file_hash",
        "local_path",
        "image_effects"
    ]

    title = schema.String(
        indexed = True,
        normalized_index = True,
        full_text_indexed = True,
        descriptive = True,
        translated = True,
        member_group = "content"
    )

    file_name = schema.String(
        required = True,
        editable = False,
        member_group = "content"
    )

    file_size = schema.Integer(
        required = True,
        editable = False,
        translate_value = lambda size, language = None, **kwargs:
            "" if size in (None, "") else format_bytes(size),
        min = 0,
        member_group = "content"
    )

    file_hash = schema.String(
        visible = False,
        searchable = False,
        text_search = False,
        member_group = "content"
    )

    local_path = schema.String(
        listed_by_default = False,
        text_search = False,
        member_group = "content"
    )

    image_effects = schema.String(
        listed_by_default = False,
        searchable = False,
        member_group = "content",
        edit_control = "woost.views.ImageEffectsEditor"
    )

    @property
    def file_extension(self):
        return os.path.splitext(self.file_name)[1]

    @event_handler
    def handle_changed(cls, e):

        if e.member is cls.local_path and e.value:
            file = e.source
            file.file_name = os.path.basename(e.value)
            file.mime_type = guess_type(e.value, strict = True)[0]
            file.file_hash = None
            file.file_size = None

    @property
    def file_path(self):

        file_path = self.local_path

        if file_path:
            if not os.path.isabs(file_path):
                file_path = app.path(file_path)
        else:
            file_path = app.path("upload", str(self.id))

        return file_path

    @classmethod
    def from_path(cls,
        path,
        dest = None,
        languages = None,
        hash = None,
        encoding = "utf-8"):
        """Imports a file into the site.

        @param path: The path to the file that should be imported.
        @type path: str

        @param dest: The base path where the file should be copied (should match
            the upload folder for the application).
        @type dest: str

        @param languages: The set of languages that the created file will be
            translated into.
        @type languages: str set

        @return: The created file.
        @rtype: L{File}
        """

        # The default behavior is to translate created files into all the languages
        # defined by the site
        if languages is None:
            languages = Language.codes

        file_name = os.path.split(path)[1]
        title, ext = os.path.splitext(file_name)

        if encoding:
            if isinstance(title, str):
                title = title.decode(encoding)
            if isinstance(file_name, str):
                file_name = file_name.decode(encoding)

        title = title.replace("_", " ").replace("-", " ")
        title = title[0].upper() + title[1:]

        file = cls()

        file.file_size = os.stat(path).st_size
        file.file_hash = hash or file_hash(path)
        file.file_name = file_name

        # Infer the file's MIME type
        mime_type = guess_type(file_name, strict = False)
        if mime_type:
            file.mime_type = mime_type[0]

        for language in languages:
            file.set("title", title, language)

        if dest is None:
            upload_path = file.file_path
        else:
            upload_path = os.path.join(dest, str(file.id))

        copy(path, upload_path)

        return file

    def make_draft(self):
        draft = Publishable.make_draft(self)

        trans = datastore.connection.transaction_manager.get()

        def copy_file(successful, source, destination):
            if successful:
                copy(source, destination)

        trans.addAfterCommitHook(
            copy_file,
            (self.file_path, draft.file_path)
        )

        return draft

    def confirm_draft(self):
        trans = datastore.connection.transaction_manager.get()

        if self.draft_source:
            def copy_file(successful, source, destination):
                if successful:
                    copy(source, destination)

            trans.addAfterCommitHook(
                copy_file,
                (self.file_path, self.draft_source.file_path)
            )

        Publishable.confirm_draft(self)

    @classmethod
    def _should_exclude_in_draft(cls, member):
        return member.name not in (
            "file_name", "file_size", "file_hash", "mime_type"
        ) and (not member.editable or not member.visible)

def file_hash(source, algorithm = "md5", chunk_size = 1024):
    """Obtains a hash for the contents of the given file.

    @param source: The file to obtain the hash for. Can be given as a file
        system path, or as a reference to a file like object.
    @type source: str or file like object

    @param algorithm: The hashing algorithm to use. Takes the same values as
        L{hashlib.new}.
    @type algorithm: str

    @param chunk_size: The size of the file chunks to read from the source, in
        bytes.
    @type chunk_size: int

    @return: The resulting file hash, in binary form.
    @rtype: str
    """
    hash = hashlib.new(algorithm)

    if isinstance(source, basestring):
        should_close = True
        source = open(source, "r")
    else:
        should_close = False

    try:
        while True:
            chunk = source.read(chunk_size)
            if not chunk:
                break
            hash.update(chunk)
    finally:
        if should_close:
            source.close()

    return hash.digest()




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			December 2008
"""
from warnings import warn
from datetime import datetime
from copy import copy
from cPickle import Pickler, Unpickler
from cStringIO import StringIO
from itertools import chain
import cherrypy
from cocktail.modeling import (
    getter,
    cached_getter,
    abstractmethod,
    ListWrapper,
    OrderedSet
)
from cocktail.events import Event, EventHub
from cocktail.pkgutils import resolve
from cocktail import schema
from cocktail.translations import translations
from cocktail.controllers import context, get_parameter, session
from cocktail.persistence import (
    PersistentObject,
    PersistentList, PersistentRelationList,
    PersistentSet, PersistentRelationSet,
    PersistentMapping, PersistentRelationMapping,
    PersistentOrderedSet, PersistentRelationOrderedSet
)
from woost.models import (
    Site,
    get_current_user,
    CreatePermission,
    ReadPermission,
    ModifyPermission,
    DeletePermission,
    ReadMemberPermission,
    ModifyMemberPermission,
    ReadTranslationPermission
)
from woost.controllers.notifications import notify_user


class EditStacksManager(object):
    """A class that manages the loading and persistence of
    L{edit stacks<EditStack>} for the active HTTP session.
    """
    _edit_stack_class = "woost.controllers.backoffice.editstack.EditStack"
    _request_param = "edit_stack"
    _session_key = "woost.controllers.backoffice: preserved_edit_stacks"
    _session_id_key = "woost.controllers.backoffice: edit_stacks_id"
    _persistent_id_prefix = "Persistent"
    __current_edit_stack = None

    expiration = 30 * 60 # seconds

    def __init__(self):
        self.__stack_map = {}

    def _dumps(self, obj):
        buffer = StringIO()
        pickler = Pickler(buffer)
        pickler.persistent_id = self.__persistent_id
        pickler.dump(obj)
        return buffer.getvalue()

    def __persistent_id(self, obj):

        if isinstance(obj, PersistentObject) and obj.is_inserted:

            full_name = obj.__class__.full_name
            if isinstance(full_name, unicode):
                full_name = str(full_name)

            return "%s-%s-%s" % (
                self._persistent_id_prefix,
                full_name,
                obj.id
            )
        else:
            return None

    def _loads(self, data):
        unpickler = Unpickler(StringIO(data))
        unpickler.persistent_load = self.__persistent_load
        return unpickler.load()

    def __persistent_load(self, key):
        if key.startswith(self._persistent_id_prefix):
            parts = key.split("-")
            persistent_type = resolve(parts[1])
            id = int(parts[2])
            return persistent_type.get_instance(id)
        else:
            raise ValueError("Wrong persistent id: " + key)

    @cached_getter
    def edit_stacks(self):
        """Obtains a mapping containing all the edit stacks for the current
        browsing session.

        @return: The mapping of stacks, indexed by their numerical id.
        @rtype: mapping of int => L{EditStack}
        """
        preserved_stacks = session.get(self._session_key)
        edit_stacks = {}

        if preserved_stacks:
            for id, entry in preserved_stacks.iteritems():
                stack = self._get_edit_stack(id)
                edit_stacks[id] = stack

        return DictWrapper(edit_stacks)

    def _get_edit_stack(self, stack_id, preserved_stacks = None):

        edit_stack = self.__stack_map.get(stack_id)

        if edit_stack is None:

            if preserved_stacks is None:
                preserved_stacks = session.get(self._session_key)

            if preserved_stacks:

                entry = preserved_stacks.get(stack_id)

                if entry is None:
                    raise WrongEditStackError(stack_id)

                last_update, stack_data = entry

                if self.expiration is not None:
                    preserved_stacks[stack_id] = (
                        datetime.now(),
                        stack_data
                    )

                edit_stack = self._loads(stack_data)
                self.__stack_map[stack_id] = edit_stack

        self._remove_expired_edit_stacks(preserved_stacks)

        return edit_stack

    def _get_current_edit_stack(self):
        if self.__current_edit_stack is None:
            self.__current_edit_stack = self.request_edit_stack

        return self.__current_edit_stack

    def _set_current_edit_stack(self, edit_stack):
        self.__current_edit_stack = edit_stack

    current_edit_stack = property(
        _get_current_edit_stack,
        _set_current_edit_stack,
        doc = """The edit stack that applies to the current context.

        @return: The current edit stack.
        @rtype: L{EditStack}
        """)

    @cached_getter
    def request_edit_stack(self):
        """
        Obtains the stack of edit operations for the current HTTP request, as
        indicated by an HTTP parameter.

        @param param: The name of the HTTP parameter that contains the unique
            identifier of the stack to obtain.

        @return: The current edit stack, or None if the "edit_stack" parameter
            is missing.
        @rtype: L{EditStack}

        @raise WrongEditStackError: Raised if the requested edit stack can't be
            found on the session.
        """
        edit_stack = None
        edit_stack_param = get_parameter(
            schema.String(self._request_param, format = r"\d+-\d+")
        )

        if edit_stack_param:
            id, step = map(int, edit_stack_param.split("-"))

            edit_stack = self._get_edit_stack(id)

            # Edit state lost
            if edit_stack is None:
                raise WrongEditStackError(id)

            # Prune the stack
            else:
                while len(edit_stack) > step + 1:
                    edit_stack.pop()

        return edit_stack

    def create_edit_stack(self):
        """Creates a new edit stack.

        @return: The new edit stack.
        @rtype: L{EditStack}
        """
        edit_stack = resolve(self._edit_stack_class)()
        edit_stack.id = session.get(self._session_id_key, 0)
        edit_stack.root_url = cherrypy.request.params.get("root_url")
        session[self._session_id_key] = edit_stack.id + 1
        self.__stack_map[edit_stack.id] = edit_stack

        self._remove_expired_edit_stacks()

        return edit_stack

    def preserve_edit_stack(self, edit_stack):
        """Stores changes to the given edit stack inside the current HTTP
        session, so it can be retrieved by later requests.

        @param edit_stack: The edit stack to preserve.
        @type edit_stack: L{EditStack}
        """
        preserved_stacks = session.get(self._session_key)

        if preserved_stacks is None:
            preserved_stacks = {}
            session[self._session_key] = preserved_stacks

        preserved_stacks[edit_stack.id] = (
            datetime.now(),
            self._dumps(edit_stack)
        )

        session.save()

    def _remove_expired_edit_stacks(
        self,
        preserved_stacks = None,
        current_time = None
    ):
        if self.expiration is None:
            return

        if preserved_stacks is None:
            preserved_stacks = session.get(self._session_key)

        if preserved_stacks is None:
            return

        if current_time is None:
            current_time = datetime.now()

        for stack_id, (last_update, stack_data) in preserved_stacks.items():
            if (current_time - last_update).seconds >= self.expiration:
                del preserved_stacks[stack_id]

        session.save()


class EditStack(ListWrapper):
    """A stack describing the context of an edit session. Allows to keep track
    of nested edit operations when dealing with related elements.

    The stack can contain two basic kind of nodes:

        * L{edit nodes<EditNode>} are associated with a single
          L{item<woost.models.items.Item>}, and store all the changes
          performed on it before they are finally saved.

        * L{relation nodes<RelationNode>} indicate a nested edit operation on a
          related item or set of items.

    The first node of a stack is always an edit node. All further nodes
    alternate their kind in succession (so the first node will be followed by a
    relation node, then another edit node, and so on).

    @ivar id: A numerical identifier for the stack. It is guaranteed to be
        unique throughout the current browser session.
    """
    id = None
    root_url = None

    def push(self, node):
        """Adds a new node to the edit stack.

        @param node: The node to add.
        @type node: L{EditNode},
            L{collection<cocktail.schema.schemacollections.Collection>}
            or L{relation<cocktail.schema.schemarelations.Relation>}
        """
        node._parent_node = self._items[-1] if self._items else None
        node._stack = self
        node._index = len(self._items)
        self._items.append(node)

    def pop(self):
        """Removes the last node from the stack and returns it.

        @return: The last node of the stack.
        @rtype: L{EditNode},
            L{collection<cocktail.schema.schemacollections.Collection>}
            or L{relation<cocktail.schema.schemarelations.Relation>}

        @raise IndexError: Raised when trying to pop a node from an empty
            stack.
        """
        node = self._items.pop()
        node._stack = None
        node._index = None
        node._parent_node = None

    def go_back(self, **params):
        """Redirects the user to the parent of the topmost node in the stack.

        If the root of the stack is reached or surpassed, the user will be
        redirected to the application's root.

        @param params: Additional query string parameters to pass to the
            destination URI.
        """
        if len(self._items) > 1:
            raise cherrypy.HTTPRedirect(self._items[-2].uri(**params))
        else:
            raise cherrypy.HTTPRedirect(
                self.root_url or context["cms"].contextual_uri(**params)
            )

    def go(self, index = -1):
        """Redirects the user to the indicated node of the edit stack.

        @param index: The position of the stack to move to.
        @type index: int
        """
        raise cherrypy.HTTPRedirect(self.uri(index))

    def uri(self, index = -1):
        """Gets the location of the given position in the stack.

        @param index: The position of the stack to get the location for.
        @type index: int

        @return: The URI for the indicated position.
        @rtype: str
        """
        warn(
            "edit_stack.uri(n) is deprecated, use edit_stack[n].uri() instead",
            DeprecationWarning,
            stacklevel = 2
        )

        if index < 0:
            index = len(self) + index

        return self[index].uri()

    def to_param(self, index = -1):
        if index < 0:
            index = len(self) + index

        return "%d-%d" % (self.id, index)

    def remove_references(self, removed_items):
        """Removes all references to the indicated items from the edit stack.

        @param removed_items: The items to remove from the stack.
        @type removed_items: L{Item<woost.models.item.Item>} sequence
        """
        # For each edit node in the stack
        for i, node in enumerate(self):
            if isinstance(node, EditNode):

                # Truncate the node and the rest of the stack if the node was
                # editing a deleted item
                if node.item in removed_items:
                    while self[-1] is not node:
                        self.pop()
                    self.pop()
                    break

                # Find all relations to persistent items
                form_data = node.form_data

                for key, member in node.item.__class__.members().iteritems():
                    if isinstance(member, schema.RelationMember) \
                    and member.is_persistent_relation:

                        # Clean up references (set to None)
                        if isinstance(member, schema.Reference) \
                        and form_data.get(key) in removed_items:
                            form_data[key] = None

                        # Clean up collections (completely remove the indicated
                        # items)
                        elif isinstance(member, schema.Collection):

                            items = form_data.get(key)
                            if items:
                                for item in removed_items:
                                    while True:
                                        try:
                                            schema.remove(items, item)
                                        except:
                                            break


class StackNode(object):
    """Base (abstract) class for the different kinds of nodes that form an
    L{edit stack<EditStack>}.
    """
    __metaclass__ = EventHub
    _stack = None
    _parent_node = None
    _index = None

    @getter
    def stack(self):
        """The edit stack that the node belongs to.
        @type: L{EditStack}
        """
        return self._stack

    @getter
    def parent_node(self):
        """The stack node that the node hangs from.
        @type: L{StackNode}
        """
        return self._parent_node

    @getter
    def index(self):
        """Gets the position of the node in its stack.
        @type: int
        """
        return self._index

    def get_ancestor_node(self, node_type, include_self = False):
        """Walks up the edit stack towards its root, looking for the first node
        of the given type.

        @param node_type: The type of node to look for.
        @type node_type: L{StackNode} subclass

        @return: The first ancestor of the node of the indicated type, or None
            if the node has no ancestors of that type.
        @rtype: L{StackNode}
        """
        node = self if include_self else self._parent_node
        while node is not None:
            if isinstance(node, node_type):
                break
            node = node._parent_node

        return node

    @abstractmethod
    def uri(self, **params):
        """Gets the URI for the stack node.

        @param params: Additional query string parameters to include in the
            produced URI.

        @return: The URI for the node.
        @rtype: unicode
        """


class EditNode(StackNode):
    """An L{edit stack<EditStack>} node, used to maintain a set of changes for
    an edited item before they are finally committed.

    @ivar translations: The list of translations defined by the item (note that
        these can change during the course of an edit operation, so that's why
        they are stored in here).
    @type translations: str list
    """
    _persistent_keys = frozenset([
        "_stack",
        "_parent_node",
        "_index",
        "_item",
        "_form_data",
        "translations",
        "section"
    ])
    _item = None
    translations = None
    section = "fields"

    saving = Event("""
        An event triggered when saving the changes contained within the node,
        after loading form data to the modified item and before the transaction
        is committed.

        @ivar user: The user that makes the changes.
        @type user: L{User<woost.models.User>}

        @ivar changeset: The change set describing the changes.
        @type changeset: L{ChangeSet<woost.models.ChangeSet>}
        """)

    committed = Event("""
        An event triggered after the changes contained within the node have
        been successfully committed to the data store.

        @ivar user: The user that makes the changes.
        @type user: L{User<woost.models.User>}

        @ivar changeset: The change set describing the changes.
        @type changeset: L{ChangeSet<woost.models.ChangeSet>}
        """)

    def __init__(self, item):
        assert item is not None
        self._item = item

    def uri(self, **params):

        if "edit_stack" not in params:
            params["edit_stack"] = self.stack.to_param(self.index)

        return context["cms"].contextual_uri(
            "content",
            str(self.item.id) if self.item.is_inserted or self.item.is_deleted else "new",
            self.section,
            **params
        )

    def __getstate__(self):

        state = {}

        for key, value in self.__dict__.iteritems():
            if key in self._persistent_keys:
                if key == "_item" and not value.is_inserted:
                    value = None
                state[key] = value

        state["content_type"] = self._item.__class__
        state["item_owner"] = self._item.owner

        if self._item.__class__.translated:
            state["item_translations"] = self._item.translations.keys()

        return state

    def __setstate__(self, state):

        content_type = state.pop("content_type", None)
        item_translations = state.pop("item_translations", None)
        item_owner = state.pop("item_owner", None)

        for key, value in state.iteritems():
            if key in self._persistent_keys:
                if key == "_item":
                    if content_type is None:
                        value = None
                    elif value is None:
                        value = content_type()
                        self.initialize_new_item(
                            value,
                            item_owner,
                            item_translations
                        )

                setattr(self, key, value)

    @getter
    def content_type(self):
        """The edited content type.
        @type: L{Item<woost.models.Item>} subclass
        """
        return self._item.__class__

    @getter
    def item(self):
        """The edited item.
        @type: L{Item<woost.models.Item>}
        """
        return self._item

    def initialize_new_item(self, item, owner = None, languages = None):

        item.owner = owner

        if item.__class__.translated:
            for language in (languages or [Site.main.default_language]):
                item.new_translation(language)

    def import_form_data(self, form_data, item):
        """Update the edited item with data gathered from the form."""
        self.form_adapter.import_object(
            form_data,
            item,
            self.form_schema,
            self.content_type
        )

        # Drop deleted translations
        if item.__class__.translated:
            user = get_current_user()

            deleted_translations = (
                set(
                    language
                    for language in item.translations
                    if user.has_permission(
                        ReadTranslationPermission,
                        language = language
                    )
                )
                - set(self.translations)
            )

            for language in deleted_translations:
                del item.translations[language]

    def export_form_data(self, item, form_data):
        """Update the edit form with the data contained on the edited item."""

        self.form_adapter.export_object(
            item,
            form_data,
            self.content_type,
            self.form_schema
        )

        # Default translations
        if self.content_type.translated:

            user = get_current_user()
            available_languages = set(
                language
                for language in item.translations
                if user.has_permission(
                    ReadTranslationPermission,
                    language = language
                )
            )

            if not self._item.translations:
                default_language = Site.main.default_language
                if default_language in available_languages:
                    self._item.new_translation(Site.main.default_language)

            self.translations = [
                language
                for language in self._item.translations.keys()
                if language in available_languages
            ]
        else:
            self.translations = []

    @cached_getter
    def form_adapter(self):
        """The data adapter used to pass data between the edited item and the
        edit form.
        @type: L{Adapter<cocktail.schema.Adapter>}
        """
        adapter = schema.Adapter()
        adapter.collection_copy_mode = self._adapt_collection
        adapter.exclude([
            member.name
            for member in self.content_type.members().itervalues()
            if self.should_exclude_member(member)
        ])
        return adapter

    def should_exclude_member(self, member):

        if not member.editable:
            return True

        if not member.visible:
            return True

        # Hide relations with relation nodes in the stack
        relation_node = self.get_ancestor_node(RelationNode)
        if relation_node and member is relation_node.member.related_end:
            return True

        user = get_current_user()

        if not user.has_permission(ReadMemberPermission, member = member):
            return True

        if not user.has_permission(ModifyMemberPermission, member = member):
            return True

        if isinstance(member, schema.RelationMember) \
        and member.is_persistent_relation:

            # Hide relations to invisible types
            if not member.related_type.visible:
                return True

            # Hide empty collections with the exclude_when_empty flag
            # set
            if (
                isinstance(member, schema.Collection)
                and member.exclude_when_empty
                and not member.select_constraint_instances(
                    parent = self.item
                )
            ):
                return True

            def class_family_permission(root, permission_type):
                return any(
                    user.has_permission(permission_type, target = cls)
                    for cls in root.schema_tree()
                )

            # Require read permission for related types
            if not class_family_permission(
                member.related_type, ReadPermission
            ):
                return True

            # Integral relation
            if (
                isinstance(member, schema.Reference)
                and member.integral
                and self.item
            ):
                # Empty: require create permission
                # Has an item: require edit or delete permission
                if self.item.get(member) is None:
                    if not class_family_permission(
                        member.type,
                        CreatePermission
                    ):
                        return True
                elif not (
                    class_family_permission(
                        member.type, ModifyPermission
                    )
                    and class_family_permission(
                        member.type, DeletePermission
                    )
                ):
                    return True

        return False

    def _adapt_collection(self, context, key, value):
        return self._copy_collection(value)

    def _copy_collection(self, collection):

        if isinstance(collection,
        (PersistentList, PersistentRelationList)):
            return list(collection)
        elif isinstance(collection,
        (PersistentMapping, PersistentRelationMapping)):
            return dict(collection.iteritems())
        elif isinstance(collection,
        (PersistentOrderedSet, PersistentRelationOrderedSet)):
            return OrderedSet(collection)
        elif isinstance(collection,
        (PersistentSet, PersistentRelationSet)):
            return set(collection)
        else:
            return copy(collection)

    @cached_getter
    def form_schema(self):
        """The schema that describes the edit form for the edited item.
        @type: L{Schema<cocktail.schema.Schema>}
        """
        form_schema = self.form_adapter.export_schema(self.content_type)
        form_schema.name = "BackOfficeEditForm"
        return form_schema

    @cached_getter
    def form_data(self):
        """The data entered into the edit form."""

        form_data = {}

        # First load: fill the form with data from the edited item
        if self is self.stack[-1]:
            self.export_form_data(self.item, form_data)

        return form_data

    def iter_errors(self):
        """Iterates over the sequence of validation errors produced by the
        current edited state.
        @type: L{ValidationError<cocktail.schema.exceptions.ValidationError>}
            sequence
        """
        return self.form_schema.get_errors(
            self.form_data,
            languages = self.translations,
            persistent_object = self.item
        )

    def iter_changes(self, source = None):
        """Iterates over the set of members that the current edit state has
        modified. Each change is expressed as a tuple containing the affected
        member and language.
        @type: (L{Member<cocktail.schema.Member>}, str) sequence
        """
        source_form_data = {}
        self.form_adapter.export_object(
            source or self._item.draft_source or self._item,
            source_form_data,
            self.content_type,
            self.form_schema
        )
        return schema.diff(
            source_form_data,
            self.form_data,
            self.form_schema
        )

    def relate(self, member, item):
        """Adds a relation between the edited item and another item.

        @param member: The member describing the relation between the two
            items. It should be the end nearer to the edited item.
        @type member: L{RelationMember<cocktail.schema.RelationMember>}

        @param item: The item to relate.
        @type item: L{Item<woost.models.item.Item>}
        """
        if isinstance(member, schema.Collection):

            collection = schema.get(self.form_data, member)

            # Editing collections with duplicate entries is not allowed
            if item in collection:
                raise ValueError(
                    "Collections with duplicate entries are not allowed")

            schema.add(collection, item)
        else:
            schema.set(self.form_data, member, item)

    def unrelate(self, member, item):
        """Breaks the relation between the edited item and one of its related
        items.

        @param member: The member describing the relation between the two
            items. It should be the end nearer to the edited item.
        @type member: L{RelationMember<cocktail.schema.RelationMember>}

        @param item: The item to unrelate.
        @type item: L{Item<woost.models.item.Item>}
        """
        if isinstance(member, schema.Collection):
            collection = schema.get(self.form_data, member)
            schema.remove(collection, item)
        else:
            schema.set(self.form_data, member, None)

    def item_saved_notification(self, is_new, change):
        """Notifies the user that the node's item has been saved.

        @param is_new: Indicates if the save operation consisted of an
            insertion (True) or an update of an existing item (False).
        @type is_new: bool

        @param change: A change object describing the save operation. Will be
            set to None when saving a draft.
        @type change: L{Change<woost.models.changeset.Change>}
        """
        item = self.item
        msg = translations(
            "woost.views.BackOfficeEditView Changes saved",
            item = item,
            is_new = is_new
        )
        transient = True

        if is_new and self.parent_node is None:
            controller = cherrypy.request.handler_chain[-1]
            msg += '. <a href="%s">%s</a>.' % (
                controller.edit_uri(item.__class__, edit_stack = None),
                translations(
                    "woost.views.BackOfficeEditView Create another"
                )
            )
            transient = False

        notify_user(msg, "success", transient)


class SelectionNode(StackNode):
    """An L{edit stack<EditStack>} node representing an item selection dialog.

    @var content_type: The type of item to select.
    @type content_type: L{Item<woost.models.item.Item>} subclass

    @var selection_parameter: The name of the HTTP parameter used to forward
        the selected item to the stack's parent node once the user confirms its
        choice.
    @type selection_parameter: str
    """
    content_type = None
    selection_parameter = None

    def uri(self, **params):

        if "edit_stack" not in params:
            params["edit_stack"] = self.stack.to_param(self.index)

        params.setdefault("type", self.content_type.full_name)
        return context["cms"].contextual_uri("content", **params)


class RelationNode(StackNode):
    """An L{edit stack<EditStack>} node, used to maintain data about an action
    affecting a relation.

    @var member: The member describing the relation that is being modified.
    @type member: L{Collection<cocktail.schema.schemacollections.Collection>}
        or L{Reference<cocktail.schema.schemareference.Reference>}
    """
    member = None

    def uri(self, **params):

        if "edit_stack" not in params:
            params["edit_stack"] = self.stack.to_param(self.index)

        return context["cms"].contextual_uri("content", **params)

    def __getstate__(self):

        state = self.__dict__.copy()
        member = state.get("member")

        if member:
            state["member"] = member.name

        return state

    def __setstate__(self, state):

        member_name = state.pop("member")

        for key, value in state.iteritems():
            setattr(self, key, value)

        content_type = self.get_ancestor_node(EditNode).content_type
        self.member = content_type[member_name]


class WrongEditStackError(Exception):
    """An exception raised when requesting an edit stack that is not stored on
    the current session.

    @ivar stack_id: The unique identifier for the requested stack.
    @type stack_id: int
    """

    def __init__(self, stack_id):
        Exception.__init__(self, "Can't find edit stack %s" % stack_id)
        self.stack_id = stack_id




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			June 2009
"""
from cocktail import schema
from cocktail.controllers.viewstate import view_state
from woost.models.item import Item
from woost.models.role import Role


class UserView(Item):

    members_order = ["title", "parameters", "roles"]
    edit_controller = \
        "woost.controllers.backoffice.userviewfieldscontroller." \
        "UserViewFieldsController"
    edit_view = "woost.views.UserViewFields"

    title = schema.String(
        descriptive = True,
        translated = True,
        required = True,
        unique = True
    )

    def _parse_parameters(form_reader, data):

        if data is None:
            return None

        params = {}

        for param in data.split("\n"):
            param = param.strip()
            if param:
                parts = param.split("=")
                if len(parts) != 2:
                    return data # Parse error
                key, value = parts
                if key in params:
                    prev_value = params[key]
                    if isinstance(prev_value, basestring):
                        params[key] = [prev_value, value]
                    else:
                        prev_value.append(value)
                else:
                    params[key] = value

        return params

    def _serialize_parameters(value):
        if value:
            return "\n".join(
                "%s=%s" % (key, value)
                if isinstance(value, basestring)
                else ("\n".join("%s=%s" % (key, x) for x in value))
                for key, value in value.iteritems()
            )
        else:
            return ""

    parameters = schema.Mapping(
        keys = schema.String(),
        edit_inline = True
    )

    del _parse_parameters
    del _serialize_parameters

    roles = schema.Collection(
        items = "woost.models.Role",
        bidirectional = True
    )

    def uri(self, **kwargs):
        params = self.parameters.copy()
        params.update(kwargs)
        params = dict((str(key), value) for key, value in params.iteritems())
        return "?" + view_state(**params)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
_cache_manager = None


def get_cache_manager():
    return _cache_manager


def set_cache_manager(cache_manager):
    global _cache_manager
    _cache_manager = cache_manager
